
/*
	file: util.c
	purpose: low level error routines and utilities.
*/

#include "defs.h" 

/* procedure calls */
extern void exit();


/*************************************************************/
/* generic error handler 
*/
void show_err(dieFlag, sModule, aMsg)
int dieFlag;
char *sModule, *aMsg;
{

	fprintf(stdout, "\n");
	fprintf(stdout, "**************************************************\n");
	fprintf(stdout, "\n");
	if (dieFlag == TRUE)
		fprintf(stdout, " Fatal Error Detected!\n");
	else
		fprintf(stdout, " Warning, Error Detected!\n");
	fprintf(stdout, "\n");
	fprintf(stdout, " Module.: %s\n", sModule);
	fprintf(stdout, " Message: %s\n", aMsg);
	fprintf(stdout, "\n");
	fprintf(stdout, "**************************************************\n");
	fprintf(stdout, "\n");

	if (dieFlag == TRUE)
		exit(1);

}	/* show_err */


/*************************************************************/
/* utility that removes blanks, tabs, newlines from a string.
** adds end of string character and returns new string len.
*/
int remove_blanks(line, len, newLine)
char *line, *newLine;
int len;
{
	register int j, count, ch;

	count = 0;
	for (j=0; j<len; j++) {
		ch = line[j];
		if ((ch != ' ') && (ch != '\t') && (ch != '\n'))
			newLine[count++] = ch;
	}

	newLine[count] = '\0';
	return(count);

} 	/* remove_blanks */



/*************************************************************/
/* converts a string of characters 'inStr' to upper case letters.
*/
void upper_case(inStr)
char *inStr;
{

	register int k, len;

	len = strlen(inStr);
	for (k=0; k<=len; k++)
		inStr[k] = AA_UC(inStr[k]);

}   /* upper_case */


/*************************************************************/
/* given a 3 character residue code, return corresponding 1 letter index
*/
int pres_3to1(res)
char *res;
{
	register int i;

	for (i=0; i<CODES; i++) {
		if (0 == strcmp(res, name3letter[i]))
			return(i);
	}
    return(ERROR_FLAG);

}   /* pres_3to1 */


/*************************************************************/
/* return the index of a secondary structure code
*/
int sres_to_ix(ch)
char ch;
{
	char code, sVal[MAX_LINE_LEN];

	code = AA_UC(ch);

    if (code == CH_B)
		return(BETA);

	if (code == CH_H)
		return(HELX);

	if ((code != CH_C) && (code != CH_T) &&
	   (code != CH_X) && (code != ' ')) {
	   sprintf(sVal, "Cannot recognize secondary structure type: %c\n", ch);
	   show_err(TRUE, "sres_to_ix", sVal);
	}

	return(COIL);

} /* sres_to_ix */


/*************************************************************/
/* given a 1 character residue code, return corresponding 3 letter
*/
char *pres_1to3(ch)
char ch;
{

	if (AA_CVALID(ch) == TRUE)
		return(name3letter[AA_IX(ch)]);

	return(ERROR_STRING);

}   /* pres_1to3 */


