
/*
	file: aa.c
	purpose: low level routines for amino acids. 
*/

#include "defs.h" 

/* procedure calls */
extern void show_err();
extern int upper_case();


/*************************************************************/
/* given a 1 character residue code, return corresponding 3 letter
*/
char *pres_1to3(ch)
char ch;
{

	if (AA_CVALID(ch) == TRUE)
		return(name3letter[AA_IX(ch)]);

	return(ERROR_STRING);

}   /* pres_1to3 */


/*************************************************************/
/* given a 3 character residue code, return corresponding 1 letter index
*/
int pres_3to1(res)
char *res;
{
	register int i;

	for (i=0; i<CODES; i++) {
		if (0 == strcmp(res, name3letter[i]))
			return(i);
	}

	return(ERROR_FLAG);

}   /* pres_3to1 */


/*************************************************************/
/* return the index of a secondary structure code 
*/
int sres_to_ix(ch)
char ch;
{
	char code, sVal[MAX_LINE_LEN];

	code = AA_UC(ch);	

	if (code == CH_B)
		return(BETA);

	if (code == CH_H)
		return(HELX);

	if (code == CH_C)
		return(COIL);

	if (code == CH_X)
		return(NO_SS);

	if (code == ' ')
		return(NO_SS);

	sprintf(sVal, "Cannot recognize secondary structure type: %c\n", ch);
	show_err(TRUE, "sres_to_ix", sVal);

}	/* sres_to_ix */


/*************************************************************/
/* convert atom name of residue to index in atom table. 
*/
int atom_to_index(rix, myAtom)
int rix;
char *myAtom;
{
	register int i, num;
	char sVal[MAX_LINE_LEN];

	if ((rix < 0) || (rix >= CODES)) {
		sprintf(sVal, "Program check, illegal index into aa table: %d\n", rix);
		show_err(TRUE, "atom_to_index", sVal);
	}

	num = aaTable[rix].atomNum;
	for (i=0; i<num; i++) {
		if (0 == strcmp(myAtom, aaTable[rix].atom[i].rName)) {
			return(i);
		}
	}

	return(ERROR_FLAG);

}	/* atom_to_index */
