
/*
	file...: print.c 
	purpose: printing results for the orb run. 

	author.: Robert Boyko
	date...: Apr 16, 1997	
	place..: University of Alberta / Department of Biochemistry
*/

#include "defs.h"

/* procedure calls */
static void print_table_header(), print_table_atom();
extern char *pres_1to3(), *get_date();


/*************************************************************/
/* print chemical shift prediction title 
*/
void print_shifts_title(fp)
FILE *fp;
{

	fprintf(fp, "!\n");
	fprintf(fp, "! Predicted shifts from orb in extended PPM format\n");
	fprintf(fp, "! Date: %s", get_date());
	fprintf(fp, "!\n");
	fprintf(fp, "! %s", seqArr[seqSelect].title);
	fprintf(fp, "!\n");
	fprintf(fp, "! Atom          Predict  Sdev");
	fprintf(fp, "  Standard Sdev   Confidence\n");
	fprintf(fp, "!\n");

}	/* print_shifts_title */


/*************************************************************/
/* print chemical shift prediction title 
*/
void print_ppm_title(fp)
FILE *fp;
{

	fprintf(fp, "!\n");
	fprintf(fp, "! Predicted shifts from orb in PPM format\n");
	fprintf(fp, "! Date: %s", get_date());
	fprintf(fp, "!\n");
	fprintf(fp, "! %s", seqArr[seqSelect].title);
	fprintf(fp, "!\n");
	fprintf(fp, "! Atom          Predict  Sdev\n");
	fprintf(fp, "!\n");

}	/* print_ppm_title */


/*************************************************************/
/* print csi output title 
*/
void print_csi_title(fp)
FILE *fp;
{

	fprintf(fp, "!\n");
	fprintf(fp, "! Input file for csi program created by orb\n");
	fprintf(fp, "! Date: %s", get_date());
	fprintf(fp, "!\n");
	fprintf(fp, "! %s", seqArr[seqSelect].title);
	fprintf(fp, "!\n");
	fprintf(fp, "    #   AA      CA      HA");
	fprintf(fp, "\n");

}	/* print_csi_title */


/*************************************************************/
/* Echo the input sequence info on output. 
*/
void print_sequence(fp, aSeq, blankFlag)
FILE *fp;
ALIGN_SEQ_REC *aSeq;
int blankFlag;
{
	register int i, ix;
	char ch, sVal[MAX_LINE_LEN];

	fprintf(fp, "\n");
	sscanf(aSeq->id, "%s", sVal);
	fprintf(fp, "Sequence: ID=%s", sVal);
	fprintf(fp, "  %s", aSeq->title);

	ix = 0;
	for (i=0; i<alignLen; i++) {
		ch = aSeq->residue[i].pRes;
		if (AA_CVALID(ch)) {
			if ((ix % RES_PER_LINE) == 0)
				fprintf(fp, "   %6d ", ix + 1);
			fprintf(fp, "%c", ch);
			ix++;
			if ((ix % RES_PER_LINE) == 0)
				fprintf(fp, "\n");
			else if (((ix % 10) == 0) && (blankFlag == TRUE))
				fprintf(fp, " ");
		}
	}
	fprintf(fp, "\n");

	fflush(fp);

}	/* print_sequence */


/*************************************************************/
/* print sequences used in our multiple alignment
*/
void print_seq_names(fp)
FILE *fp;
{
	register int i;

	if (numSeq > 1) {
		fprintf(fp, "\n");
		fprintf(fp, "Comparison Sequences\n");
		fprintf(fp, "********************\n");
		fprintf(fp, "\n");
		fprintf(fp, "      ID  Title\n");
		for (i=0; i<numSeq; i++) {
			fprintf(fp, "%8s", seqArr[i].id);
			fprintf(fp, "  %s", seqArr[i].title);
		}
	}

}	/* print_seq_names */


/*************************************************************/
/* verbose output predict amino acid homology table 
*/
void print_table(fp, pRec, rix, atomNum)
FILE *fp;
int rix, atomNum;
PREDICT_REC *pRec;
{
	register int k;
	PREDICT_REC *p;
	char unpredict[MAX_LINE_LEN];

	strcpy(unpredict, "");
	print_table_header(fp, rix);
	for (k=0; k<atomNum; k++) {
		p = &pRec[k];
		if (p->dswVal != NO_SHIFT_VAL)
			print_table_atom(fp, p);
		else {
			strcat(unpredict, p->atom);
			strcat(unpredict, " ");
		}
	}
	if (strlen(unpredict) > 0) {
		fprintf(fp, "  Atoms not predicted, Wishart shifts unavailable.\n");
		fprintf(fp, "     %s\n", unpredict);
	}

}	/* print_table */


/*************************************************************/
/* log chemical shift prediction table title
*/
static void print_table_atom(fp, p)
FILE *fp;
PREDICT_REC *p;
{
	register int i, j;
	char ssa;
	HOMOL_REC *h;

	fprintf(fp, " %5s", p->atom);
	fprintf(fp, " %7.2f", p->predict);
	fprintf(fp, "  Tables                              ");
	fprintf(fp, " %7.2f %7.1f%%\n", p->dswVal, 100.0 * p->dswWt);

	for (j=0; j<p->homNum; j++) {
		h = &p->hom[j];
		for (i=0; i<2; i++) {
			if (h->ppm[i] != NO_SHIFT_VAL) {
				if (h->ssaType == 0)
					ssa = 'T'; 
				else
					ssa = 'F';
				fprintf(fp, " %21s %5s   %c", h->seqId, h->atom, ssa);
				fprintf(fp, " %7.2f %4d", h->ppm[i], h->molScor);
				if (h->shift[i] == NO_SHIFT_VAL)
					fprintf(fp, "   xxxx    xxxx");
				else {
					fprintf(fp, " %6.2f %7.2f", 
						h->shift[i] - p->dswVal, h->shift[i]);
				}
				if (h->selectShift == i)
					fprintf(fp, " %7.1f%%", 100.0 * h->homWt);
				else
					fprintf(fp, " %7.1f%%", 0.0);
				putc('\n', fp);
			}
		}
	}

	putc('\n', fp);
	fflush(fp);

}	/* print_table_atom */


/*************************************************************/
/* print header for homology table 
*/
static void print_table_header(fp, rix)
FILE *fp;
int rix;
{
	register int j, ch;

	fprintf(fp, "\n");
	fprintf(fp, "*****************************************************************\n");
	fprintf(fp, "\n");
	for (j=0; j<MAX_ID_LEN-4; j++)
		putc(' ', fp);
	fprintf(fp, " SeqID   AA-ID  SS  Global Local  Struct\n");
	for (j=0; j<MAX_ID_LEN-4; j++)
		putc(' ', fp);
	fprintf(fp, "                    Homol  Homol   Homol\n");
	for (j=0; j<MAX_ID_LEN-4; j++)
		putc(' ', fp);
	fprintf(fp, " -----   -----  --  -----  -----   -----\n");

	for (j=0; j<numSeq; j++) {

		if (j == seqSelect)
			fprintf(fp, "       Predict");
		else
			fprintf(fp, "       Homolog");
			
		fprintf(fp, "%8s ", seqArr[j].id);

		ch = seqArr[j].residue[rix].pRes;
		if (TRUE == AA_CVALID(ch)) {

			fprintf(fp, "  %c-", seqArr[j].residue[rix].pRes);
			fprintf(fp, "%-4d", seqArr[j].residue[rix].idNum); 
			fprintf(fp, "  %c ", seqArr[j].residue[rix].sRes); 
	
			if (j != seqSelect) {
				fprintf(fp, " %5d", seqArr[j].homGlobScor);
				fprintf(fp, " %6d", seqArr[j].residue[rix].homLocScor);
				fprintf(fp, " %7d", seqArr[j].residue[rix].homStructScor);
			}
		}

		fprintf(fp, "\n");
	}

	fprintf(fp, "\n");
	fprintf(fp, "  Atom Predict  Source  Atom SSA  PPMVal Scor Offset   Shift    PctWt\n");
	fprintf(fp, "  ---- -------  ------  ---- ---  ------ ---- ------   -----    -----\n");
	fflush(fp);

}	/* print_table_header */


/*************************************************************/
/* print one chemical shift prediction 
*/
void print_ppm(fp, pRec, scpRes, sIdNum, atomNum)
FILE *fp;
char scpRes;
PREDICT_REC *pRec;
int sIdNum, atomNum;
{
	register int k;
	PREDICT_REC *p;
	char label[MAX_LINE_LEN];

	for (k=0; k<atomNum; k++) {
		p = &pRec[k];
		if ((p->predict != NO_SHIFT_VAL) && (p->dswVal != NO_SHIFT_VAL)) {
			sprintf(label, "1:%s_%d:%-5s", pres_1to3(scpRes), sIdNum, p->atom);
			fprintf(fp, "%-15s %7.2f %5.2f", label, p->predict, p->predStd);
			fprintf(fp, "\n");
		}
	}

}	/* print_ppm */


/*************************************************************/
/* print one chemical shift prediction 
*/
void print_shift(fp, pRec, scpRes, sIdNum, atomNum)
FILE *fp;
char scpRes;
PREDICT_REC *pRec;
int sIdNum, atomNum;
{
	register int k;
	PREDICT_REC *p;
	char label[MAX_LINE_LEN];
	double sdev;


	for (k=0; k<atomNum; k++) {
		p = &pRec[k];
		if ((p->predict != NO_SHIFT_VAL) && (p->dswVal != NO_SHIFT_VAL)) {
			sprintf(label, "1:%s_%d:%-5s", pres_1to3(scpRes), sIdNum, p->atom);
			fprintf(fp, "%-15s %7.2f %5.2f", label, p->predict, p->predStd);
			fprintf(fp, "  %7.2f %5.2f", p->dswVal, p->dswStd);
			fprintf(fp, "       ");

			if (p->predStd > 0.0) {
				sdev = p->dswStd / p->predStd;
				if (sdev <= 1.0)
					fprintf(fp, "-   ");
				else if (sdev <= 3.0)
					fprintf(fp, "*   ");
				else if (sdev <= 5.0)
					fprintf(fp, "**  ");
				else if (sdev <= 9.0)
					fprintf(fp, "*** ");
				else
					fprintf(fp, "****");
			}
			else fprintf(fp, "****");

			fprintf(fp, "\n");
		}
	}

}	/* print_shift */


/*************************************************************/
/* print one chemical shift prediction for csi data file
*/
void print_csi_shift(fp, pRec, pch, sIdNum, atomNum)
FILE *fp;
PREDICT_REC *pRec;
char pch;
int sIdNum, atomNum;
{
	register int k;
	double calpha, halpha;
	PREDICT_REC *p;

	calpha = 0.0;
	halpha = 0.0;

	for (k=0; k<atomNum; k++) {
		p = &pRec[k];
		if (0 == strcmp(p->atom, "CA"))
			calpha = p->predict;
		else if (0 == strcmp(p->atom, "HA"))
			halpha = p->predict;
		else if (0 == strcmp(p->atom, "HA1"))
			halpha = p->predict;
		else if (0 == strcmp(p->atom, "HA2"))
			halpha = 0.5 * (halpha + p->predict);
	}

	fprintf(fp, "%5d    %c %7.2f %7.2f\n", sIdNum, pch, calpha, halpha);

}  /* print_csi_shift */


/*************************************************************/
/* print separation line between different data 
*/
/*
void print_divide(fp)
FILE *fp;
{
	register int i;

	fprintf(fp, "\n");
	for (i=0; i<56; i++)
		putc('.', fp);
	fprintf(fp, "\n\n");

print_divide */
