
/*
	file: ralign.c
	reads input alignment for orb program	
*/

#include "defs.h"

/* static vars */
static int sResNum[MAX_SEQ_NUM];
static int pResNum[MAX_SEQ_NUM];
static int sMap[MAX_SEQ_NUM][MAX_SEQ_LEN];
static int pMap[MAX_SEQ_NUM][MAX_SEQ_LEN];
static int rMap[MAX_SEQ_NUM][MAX_SEQ_LEN];

/* procedure calls */
static FILE *read_xalign_fname();
static void read_keyword(), read_seq_titles();
static void read_xalign_data(), s_read_align(), check_xalign_seq();
static void calc_end_points(), calc_residue_idNum(), read_id_nums();
static int read_xalign_data_group(), p_read_align();
extern void show_err();
extern char *strtok();


/*************************************************************/
/* reads xalign data file.
*/
void read_xalign()
{
	FILE *fp;
	 
	numSeq = 0;

	fp = read_xalign_fname();
	read_seq_titles(fp);
	read_xalign_data(fp);

	calc_end_points();
	calc_residue_idNum();

	fclose(fp);

}	/* read_xalign */


/*************************************************************/
/* opens xalign data file.
*/
static FILE *read_xalign_fname()
{
	FILE *fp;
	char sVal[MAX_LINE_LEN], dataFname[MAX_LINE_LEN];
	 
	fprintf(stdout, "\n");
	fprintf(stdout, " Orb assumes you already have an aligned set of two\n");
	fprintf(stdout, " or more sequences. If you do not, run the xalign\n");
	fprintf(stdout, " program to create this file. Make sure you are\n");
	fprintf(stdout, " satisfied with the xalign output before proceeding\n");
	fprintf(stdout, " from here.\n");
	fprintf(stdout, "\n");
	fprintf(stdout, " Please enter your alignment data file.\n");
	fprintf(stdout, PROG_PROMPT);

	fscanf(stdin, "%s", dataFname);
	fp = (FILE *)fopen(dataFname, "r");
	if (fp == NULL) {
		sprintf(sVal, "Cannot open alignment dataFile: %s\n", dataFname);
		show_err(TRUE, "read_xalign_fname", sVal);
	}

	fprintf(stdout, "\n Reading alignment data file: %s\n", dataFname);

	return(fp);

}	/* read_xalign_fname */


/*************************************************************/
/* reads number of sequences and the names of the sequences. 
*/
static void read_seq_titles(fp)
FILE *fp;
{
	int id, i, len;
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	read_keyword(fp, TITLE_KEY);

	i = 0;
	while (TRUE) {
		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Unexpected EOF, is this an alignment file?\n");
			show_err(TRUE, "read_seq_titles", sVal);
		}
		if (strcmp(aStr, "1") == 0)
			break;
		if (i > 10) {
			sprintf(sVal, "Could not find sequence #1, check align data\n");
			show_err(TRUE, "read_seq_titles", sVal);
		}
		i++;
	}

	while (TRUE) {

/* process Num field */

		id = atoi(aStr);
		if (id != numSeq+1) {
			sprintf(sVal, "Found sequence num %s, expecting %d\n",
				aStr, numSeq+1);
			show_err(TRUE, "read_seq_titles", sVal);
		}
		id--;

/* process seq-id field */

		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Unexpected EOF when reading seq-id\n");
			show_err(TRUE, "read_seq_titles", sVal);
		}

		if (strlen(aStr) > MAX_ID_LEN) {
			sprintf(sVal, "Illegal Seq-ID: %s MaxLen = %d\n", aStr, MAX_ID_LEN);
			show_err(TRUE, "read_seq_titles", sVal);
		}
		strcpy(seqArr[numSeq].id, aStr);

/* process sequence title field */

		fgets(sVal, MAX_LINE_LEN, fp);
		len = strlen(sVal);
		for (i=0; i<len; i++) {
			if ((sVal[i] != ' ') && (sVal[i] != '\t'))
				break;
		}
		if (sVal[i] == '\n') {
			sprintf(sVal, "Missing sequence title at: %s\n", seqArr[numSeq].id);
			show_err(TRUE, "read_seq_titles", sVal);
		}
		strcpy(seqArr[numSeq].title, &sVal[i]);

/* process residue labelling field if given */

		read_id_nums(sVal, rMap[numSeq]);

		numSeq++;

/* read next sequence line */

		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Unexpected EOF, Cannot find key: %s\n", STATS_KEY);
			show_err(TRUE, "read_seq_titles", sVal);
		}

		else if ((0 == strcmp(aStr, PCT_TABLE_KEY)) ||
				 (0 == strcmp(aStr, STATS_KEY)))
			break;
	}

}	/* read_seq_titles */


/*************************************************************/
/* read the sequence alignment 
*/
static void read_xalign_data(fp)
FILE *fp;
{
	register int i, len;
	char sVal[MAX_LINE_LEN];
	
	read_keyword(fp, ALIGN_KEY);

	alignLen = 0;
	for (i=0; i<numSeq; i++) {
		pResNum[i] = 0;
		sResNum[i] = 0;
	}

	fgets(sVal, MAX_LINE_LEN, fp);
	fgets(sVal, MAX_LINE_LEN, fp);

	do {

		len = read_xalign_data_group(fp);
		alignLen += len;

	} while (len > 0);

	for (i=0; i<numSeq; i++)
		check_xalign_seq(i);

}	/* read_xalign_data */


/*************************************************************/
/* check the alignment which we have read in for one sequence. 
*/
static void check_xalign_seq(ix)
int ix;
{
	register int j;
	char sVal[MAX_LINE_LEN];
	
	if (sResNum[ix] != 0) {

		if (pResNum[ix] != sResNum[ix]) {
			sprintf(sVal, "Primary/secondary structure misaligned: %s\n",
				seqArr[ix].id);
			show_err(TRUE, "check_xalign_seq", sVal);
		}

		for (j=0; j<pResNum[ix]; j++) {
			if (pMap[ix][j] != sMap[ix][j]) {
				sprintf(sVal, "Misaligned primary/secondary structure: %s\n",
					seqArr[ix].id);
				show_err(TRUE, "read_xalign_data", sVal);
			}
		}

	}

	else {
		for (j=0; j<alignLen; j++)
			seqArr[ix].residue[j].sRes = CH_X;
	}

	seqArr[ix].numRes = pResNum[ix];

}	/* check_xalign_seq */


/*************************************************************/
/* read one group in the sequence alignment. 
*/
static int read_xalign_data_group(fp)
FILE *fp;
{
	int seqIx, kLen, pLineLen, pLineMax, structFlag;
	char key[MAX_LINE_LEN], sVal[MAX_LINE_LEN];
	
	seqIx = pLineMax = 0;
	
	while (TRUE) {

		/* check for end of data */
		if (EOF == fscanf(fp, "%s", key)) {
			if (seqIx == 0)
				return(0);
			sprintf(sVal, "Unexpected EOF in aligned sequences\n");
			show_err(TRUE, "read_xalign_data_group", sVal);
		}
		kLen = strlen(key) - 1;

		/* read primary sequence line */
		if ((seqIx < numSeq) && (0 == strncmp(key, seqArr[seqIx].id, kLen))) {
			pLineLen = p_read_align(fp, seqIx, alignLen);
			pLineMax = MAX(pLineLen, pLineMax);
			seqIx++;
			structFlag = TRUE;
		}

		/* read structure line if it exists */
		else if ((seqIx <= numSeq) && (0 == strncmp(key, S_TITLE_KEY, kLen))) {
			if (structFlag == FALSE) {
				sprintf(sVal, "Found consecutive struct lines in alignment\n");
				show_err(TRUE, "read_xalign_data_group", sVal);
			}
			s_read_align(fp, seqIx-1, alignLen);
			structFlag = FALSE;
		}

		/* have we reached the consensus line? */
		else if (0 == strncmp(key, CONSENSUS_KEY, kLen)) {
			if (seqIx != numSeq) {
				sprintf(sVal, "Consensus sequence line unexpectedly found\n");
				show_err(TRUE, "read_xalign_data_group", sVal);
			}
			fgets(sVal, MAX_LINE_LEN, fp);
			return(pLineMax);
		}

		else {
			sprintf(sVal, "Unexpected sequence ID found: %s\n", key);
			show_err(TRUE, "read_xalign_data_group", sVal);
		}

	}

}	/* read_xalign_data_group */


/*************************************************************/
/* scans xalign input file for keyword
*/
static void read_keyword(fp, key)
FILE *fp;
char *key;
{
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	while (TRUE) {

		if (EOF == fscanf(fp, "%s", aStr)) {
			sprintf(sVal, "Cannot find keyword in xalign input: %s\n", key);
			show_err(TRUE, "read_keyword", sVal);
		}

		else if (0 == strcmp(aStr, key))
			break;

		fgets(aStr, MAX_LINE_LEN, fp);
	}

}	/* read_keyword */



/*************************************************************/
/* read primary sequence line from xalign output 
** return length of the line
*/
static int p_read_align(fp, ix, aLen)
FILE *fp;
int ix, aLen;
{
	register int count;
	int len;
	char ch, uch, sVal[MAX_LINE_LEN];

	len = 0;
	count = 0;

	ch = getc(fp);
	if (ch != ' ') {
		sprintf(sVal, "Expecting ' ' after ':' but found '%c' instead\n", ch);
		show_err(TRUE, "p_read_align", sVal);
	}

	while (TRUE) {
		ch = getc(fp);
		uch = AA_UC(ch);
		if (((ch >= '0') && (ch <= '9')) || (ch == '\n')) {
			if (ch != '\n')
				fgets(sVal, MAX_LINE_LEN, fp);
			break;
		}

		if (ch == ' ') {
			seqArr[ix].residue[aLen].pRes = ' ';
		}
		else if (TRUE == AA_CVALID(uch)) {
			seqArr[ix].residue[aLen].pRes = uch;
			pMap[ix][pResNum[ix]++] = aLen;
			len = count + 1;
		}
		else {
			sprintf(sVal, "Illegal char in alignment: '%c'\n", uch);
			show_err(TRUE, "p_read_align", sVal);
		}

		count++;
		aLen++;
	}

	return(len);

}	/* p_read_align */


/*************************************************************/
/* read structure line from xalign output 
*/
static void s_read_align(fp, ix, aLen)
FILE *fp;
int ix, aLen;
{
	char ch, uch, sVal[MAX_LINE_LEN];

	fscanf(fp, "%c", &ch);
	if ((ch != ' ') && (ch != '\n')) {
		sprintf(sVal, "Expecting Structure ': ' but found ':%c' instead\n", ch);
		show_err(TRUE, "s_read_align", sVal);
	}

	while (TRUE) {
		ch = getc(fp);
		uch = AA_UC(ch);
		if (((ch >= '0') && (ch <= '9')) || (ch == '\n')) {
			if (ch != '\n')
				fgets(sVal, MAX_LINE_LEN, fp);
			break;
		}

		if (ch == ' ') {
			seqArr[ix].residue[aLen].sRes = ' ';
		}
		else if (TRUE == AA_SVALID(uch)) {
			seqArr[ix].residue[aLen].sRes = uch;
			sMap[ix][sResNum[ix]++] = aLen;
		}
		else {
			sprintf(sVal, "Illegal secondary structure char: '%c'\n", uch);
			show_err(TRUE, "s_read_align", sVal);
		}

		aLen++;
	}

}	/* s_read_align */


/*************************************************************/
/* calc endpoints for the various sequences for homology calc. 
*/
static void calc_end_points()
{
	register int j, k;
	char ch1;

	for (j=0; j<numSeq; j++) {
		for (k=0; k<alignLen; k++) {
			ch1 = seqArr[j].residue[k].pRes;
			if (AA_CVALID(ch1) == TRUE) {
				seqArr[j].startIx = k;
				break;
			}
		}
		for (k=alignLen-1; k>0; k--) {
			ch1 = seqArr[j].residue[k].pRes;
			if (AA_CVALID(ch1) == TRUE) {
				seqArr[j].endIx = k;
				break;
			}
		}
	}

}	/* calc_end_points */


/*************************************************************/
/* calculate residue id numbers for each sequence in mult align. 
*/
static void calc_residue_idNum()
{
	register int j, k, ch, mCount;
	ALIGN_SEQ_REC *seqPtr;
	int *resMap;

	for (j=0; j<numSeq; j++) {
		seqPtr = &seqArr[j];
		resMap = rMap[j];
		mCount = 0;
		if (resMap[0] == MIN_INT) {
			for (k=0; k<seqPtr->numRes; k++)
				resMap[k] = k+1;
		}
		for (k=0; k<alignLen; k++) {
			ch = seqPtr->residue[k].pRes;
			if (TRUE == AA_CVALID(ch))
				seqPtr->residue[k].idNum = resMap[mCount++];
			else
				seqPtr->residue[k].idNum = MIN_INT;
		}
	}

}	/* calc_residue_idNum */


/*************************************************************/
/* Reads the residue id numbers
*/
static void read_id_nums(title, aMap)
char *title;
int *aMap;
{
	register int i, k, rCount, nd;
	int startId, endId;
	char *p;

	aMap[0] = MIN_INT;
	rCount = 0;
	p = strchr(title, OPEN_RES_ID);
	while (p != NULL) {
		nd = sscanf(&p[1], "%d-%d]", &startId, &endId);
		if (nd == 2) {
			for (k=startId; k<=endId; k++) {
				aMap[rCount++] = k;
			}
		}
		p = strchr(&p[1], OPEN_RES_ID);
	}

}	/* read_id_nums */
