

/*
	file: rdsw.c
	purpose: reads wishart's shifts database 
*/

#include "defs.h" 

/* global vars */
char	dswFname[MAX_FNAME_LEN];

/* procedure calls */
static void read_dsw_shift_file();
extern void show_err();
extern int pres_3to1();


/*************************************************************/
/* Read dsw shift parameters.
*/
void read_dsw_shifts(parmsfp)
FILE *parmsfp;
{
	FILE *fp;
	char sVal[MAX_LINE_LEN];

/* read and process each shift file */
	if (read_prompt(parmsfp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot read dsw shift file\n");
		show_err(TRUE, "read_dsw_shifts", sVal);
	}
	fscanf(parmsfp, "%s\n", dswFname);
	fp = fopen(dswFname, "r");
	if (fp == NULL) {
		sprintf(sVal, "Cannot open dsw shifts file: %s\n", dswFname);
		show_err(TRUE, "read_dsw_shifts", sVal);
	}
	fprintf(stdout, "Reading Wishart shift file: %s\n", dswFname);
	read_dsw_shift_file(fp);
	fclose(fp);

}	/* read_dsw_shifts */

/*************************************************************/
/* Read Wishart default shifts file 
*/
void read_dsw_shift_file(fp)
FILE *fp;
{
	int i, j, nd, ix, num;
	double coil, helx, beta, ss;
	double sdCoil, sdHelx, sdBeta, sdss;
	char res[MAX_LINE_LEN], atom[MAX_LINE_LEN];
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	while (TRUE) {
		if (NULL == fgets(aStr, MAX_LINE_LEN, fp)) {
			break;
		}
		if (aStr[0] != COMMENT) {
			nd = sscanf(aStr, 
				"%3s:%s %*s %lf%lf %*s %lf%lf %*s %lf%lf %*s %lf%lf",
				res, atom, &coil, &sdCoil, &beta, &sdBeta, &helx, &sdHelx,
				&ss, &sdss);
			if (nd != 10) {
				sprintf(sVal, "Bad file format: %s\n", aStr);
				show_err(FALSE, "read_dsw_shift_file", sVal);
				continue;
			}

			ix = pres_3to1(res);
			if (ix < 0) {
				sprintf(sVal, "Illegal amino acid: %s\n", res);
				show_err(TRUE, "read_dsw_shift_file", sVal);
			}

			strcpy(aaTable[ix].rName, res);
			num = aaTable[ix].atomNum;
			for (j=0; j<num; j++) {
				if (0 == strcmp(atom, aaTable[ix].atom[j].rName)) {
					break;
				}
			}
			if (j == num) {
				sprintf(sVal, "Unable to recognize atomname: %s\n", aStr);
				show_err(FALSE, "read_dsw_shift_file", sVal);
			}

			strcpy(aaTable[ix].atom[j].rName, atom);
			aaTable[ix].atom[j].dswShift[COIL] = coil;
			aaTable[ix].atom[j].dswStddev[COIL] = sdCoil;
			aaTable[ix].atom[j].dswShift[BETA] = beta;
			aaTable[ix].atom[j].dswStddev[BETA] = sdBeta;
			aaTable[ix].atom[j].dswShift[HELX] = helx;
			aaTable[ix].atom[j].dswStddev[HELX] = sdHelx;
			aaTable[ix].atom[j].dswShift[NO_SS] = ss;
			aaTable[ix].atom[j].dswStddev[NO_SS] = sdss;
			aaTable[ix].atomNum++;
			if (aaTable[ix].atomNum >= MAX_RES_ATOM_NUM) {
				sprintf(sVal, "Max atoms %d exceeded for: %s\n",
					MAX_RES_ATOM_NUM, aaTable[ix].rName);
				show_err(TRUE, "read_dsw_shifts", sVal);
			}
		}
	}
}	/* read_dsw_shifts */
