
/*
	file: rparms.c
	purpose: reading numeric fields in the parameter file. 
*/

#include "defs.h" 

/* global vars */
double	csGlobMin, csLocMin, csStructMin, csMolMin;
double	csLocWind, csStructWind, csDbase;

/* procedure calls */
static void read_cs_dbase(), read_cs_min(), read_cs_wind();
extern void show_err();


/*************************************************************/
/* read weighting parms for calculating chemical shift 
*/
void read_cs_parms(parmsfp)
FILE *parmsfp;
{

	fprintf(stdout, "Reading weighting constants...\n");
	read_cs_min(parmsfp);
	read_cs_wind(parmsfp);
	read_cs_dbase(parmsfp);

}	/* read_cs_parms */


/*************************************************************/
/* read chemical shift prediction db shift weight
*/
static void read_cs_dbase(fp)
FILE *fp;
{
	int nd;
	char sVal[MAX_LINE_LEN];

	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find shift database weighting prompt\n");
		show_err(TRUE, "read_cs_dbase", sVal);
	}
	nd = fscanf(fp, "%lf", &csDbase);
	if (nd != 1) {
		sprintf(sVal, "Cannot find shift database weighting entry\n");
		show_err(TRUE, "read_cs_dbase", sVal);
	}
	if (csDbase < 0.0) {
		sprintf(sVal, "Shift database weighting cannot be < 0.0\n");
		show_err(TRUE, "read_cs_dbase", sVal);
	}
	csDbase = csDbase * 5.0;

}	/* read_cs_dbase */


/*************************************************************/
/* read window sizes for local homologies 
*/
static void read_cs_wind(fp)
FILE *fp;
{
	int nd;
	char sVal[MAX_LINE_LEN];

/* local homology window size */
	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find local homology window size prompt\n");
		show_err(TRUE, "read_cs_wind", sVal);
	}
	nd = fscanf(fp, "%lf", &csLocWind);
	if (nd != 1) {
		sprintf(sVal, "Cannot find local homology window size entry\n");
		show_err(TRUE, "read_cs_wind", sVal);
	}
	if (csLocWind <= 0.0) {
		sprintf(sVal, "Local homology window size cannot be < 1\n");
		show_err(TRUE, "read_cs_wind", sVal);
	}

/* structural homology window size */
	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find structural homology window size prompt\n");
		show_err(TRUE, "read_cs_wind", sVal);
	}
	nd = fscanf(fp, "%lf", &csStructWind);
	if (nd != 1) {
		sprintf(sVal, "Cannot find structural homology window size entry\n");
		show_err(TRUE, "read_cs_wind", sVal);
	}
	if (csStructWind <= 0.0) {
		sprintf(sVal, "Structural homology window size cannot be < 1\n");
		show_err(TRUE, "read_cs_wind", sVal);
	}

}	/* read_cs_wind */


/*************************************************************/
/* minimum homology values 
*/
static void read_cs_min(fp)
FILE *fp;
{
	int nd;
	char sVal[MAX_LINE_LEN];

/* molecular homology */
	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find minimum molecular homology prompt\n");
		show_err(TRUE, "read_cs_mol_hom", sVal);
	}
	nd = fscanf(fp, "%lf", &csMolMin);
	if (nd != 1) {
		sprintf(sVal, "Cannot find minimum molecular homology entry\n");
		show_err(TRUE, "read_cs_mol_hom", sVal);
	}
	if (csMolMin < 0.0) {
		sprintf(sVal, "Minimum Molecular homology cannot be < 0.0\n");
		show_err(TRUE, "read_cs_mol_hom", sVal);
	}

/* global homology */
	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find minimum global homology prompt\n");
		show_err(TRUE, "read_cs_min", sVal);
	}
	nd = fscanf(fp, "%lf", &csGlobMin);
	if (nd != 1) {
		sprintf(sVal, "Cannot find minimum global homology entry\n");
		show_err(TRUE, "read_cs_min", sVal);
	}
	if (csGlobMin < 0.0) {
		sprintf(sVal, "Minimum Global homology cannot be < 0.0\n");
		show_err(TRUE, "read_cs_min", sVal);
	}

/* local homology */
	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find minimum local homology prompt\n");
		show_err(TRUE, "read_cs_min", sVal);
	}
	nd = fscanf(fp, "%lf", &csLocMin);
	if (nd != 1) {
		sprintf(sVal, "Cannot find minimum local homology entry\n");
		show_err(TRUE, "read_cs_min", sVal);
	}
	if (csLocMin < 0.0) {
		sprintf(sVal, "Minimum Local homology cannot be < 0.0\n");
		show_err(TRUE, "read_cs_min", sVal);
	}

/* structural homology */
	if (read_prompt(fp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find minimum structural homology prompt\n");
		show_err(TRUE, "read_cs_min", sVal);
	}
	nd = fscanf(fp, "%lf", &csStructMin);
	if (nd != 1) {
		sprintf(sVal, "Cannot find minimum structural homology entry\n");
		show_err(TRUE, "read_cs_min", sVal);
	}
	if (csStructMin < 0.0) {
		sprintf(sVal, "Minimum Structural homology cannot be < 0.0\n");
		show_err(TRUE, "read_cs_min", sVal);
	}

}	/* read_cs_min */
