
/*
	This file contains routines required to perform star2ppm
*/

#include "main.h"

/* procedure calls */
static void proc_atoms();
static FILE *open_data_file(), *open_output_file();
static int read_sequence(), read_convert(), valid_atom_name();
static AMINO_ACID_REC aaTable[CODES];

extern void upper_case(), show_err();
extern char pres_3to1();


/*************************************************************/
/* convert shift file 
*/
void do_star2ppm()
{
	int seqNum, cSize;
	char sequence[MAX_SEQ_NUM];
	FILE *outfp, *shiftsfp;

	shiftsfp = open_data_file(dataFname, "shifts");
	fprintf(stdout, "\n Shifts File: %s\n", dataFname);
	seqNum = read_sequence(shiftsfp, sequence);
	if (seqNum < 0) {
		fprintf(stdout, "\n Incorrect format for star2ppm. \n");
		fprintf(stdout, " Maybe you have a star file without the chemical\n");
		fprintf(stdout, " shift value error specified.... \n");
		exit(-1);
	}
	fprintf(stdout, " Number of amino acids in sequence: %d\n\n", seqNum);
	rewind(shiftsfp);

	cSize = read_convert();
	fprintf(stdout, "\n Conversion Table: %s\n", convertFname);
	fprintf(stdout, " Number of table entries: %d\n", cSize);
	fprintf(stdout, " Make sure that this shift table is up to date.\n");
	fprintf(stdout, " \n");

	outfp = open_output_file(resultsFname);
	fprintf(stdout, "\n Output File: %s\n\n", resultsFname);
	fprintf(outfp, "#\n");
	fprintf(outfp, "# This file generated by star2ppm from %s\n", dataFname);
	fprintf(outfp, "#\n");

	proc_atoms(shiftsfp, outfp);
	fprintf(stdout, " Done\n\n");

}	/* do_star2ppm */


/*************************************************************/
/* opens user data file.
*/
static FILE *open_data_file(fname, ftype)
char *fname, *ftype;
{
	FILE *fp;

	fp = fopen(fname, "r");
	while (fp == NULL) {
		fprintf(stdout, "\n\n");
		fprintf(stdout, " Please enter your %s file.\n", ftype);
		fprintf(stdout, "\n");
		fscanf(stdin, "%s", fname);
		fp = fopen(fname, "r");
	}

	return(fp);

}	/* open_data_file */


/*************************************************************/
/* opens output file.
*/
static FILE *open_output_file(fname)
char *fname;
{
	FILE *fp;

	fp = fopen(fname, "w");
	while (fp == NULL) {
		fprintf(stdout, "\n\n");
		fprintf(stdout, " Enter your output file.\n");
		fprintf(stdout, "\n");
		fscanf(stdin, "%s", fname);
		fp = fopen(fname, "w");
	}

	return(fp);

}	/* open_output_file */


/*************************************************************/
/* Read pipe data file for the sequence
*/
static int read_sequence(fp, mySeq)
FILE *fp;
char *mySeq;
{
	int seqLen;
	char res3[MAX_LINE_LEN], sVal[MAX_LINE_LEN];
	char ch, key[MAX_LINE_LEN], atom[MAX_LINE_LEN];
	int nd, id, nextId, lo, hi, stereo, line;
	double shift;

	nextId = 0;
	lo = 100000;
	hi = 0;
	while (TRUE) {

		nd = fscanf(fp, "%d %d %s %s %*s %lf %*s %d",
			&line, &id, res3, atom, &shift, &stereo);

		if (nd == 6) {
			lo = MIN(lo, id);
			hi = MAX(hi, id);
			ch = pres_3to1(res3);
			if (ch == 'X') {
				sprintf(sVal, "Illegal amino acid '%s' in file %s", 
					res3, dataFname);
				show_err(stdout, FALSE, "read_sequence", sVal);
				RC = 1;
			}
			mySeq[id] = ch;
		}

		if (NULL == fgets(sVal, MAX_LINE_LEN, fp))
			break;
	}

	seqLen = hi - lo + 1;
	return(seqLen);

}   /* read_sequence */


/*************************************************************/
/* Read shifts conversion file 
*/
static int read_convert()
{
	char res[MAX_LINE_LEN], sVal[MAX_LINE_LEN], old[MAX_LINE_LEN];
	char n1[MAX_LINE_LEN], n2[MAX_LINE_LEN], n3[MAX_LINE_LEN];
	int nd, i, j, num, amb, ix;
	FILE *fp;

	fp = fopen(convertFname, "r");
	while (fp == NULL) {
		fprintf(stdout, "\n\n");
		fprintf(stdout, " I did not find a 'star2ppm.table' file in\n");
		fprintf(stdout, " this directory. Please enter a valid star2ppm\n");
		fprintf(stdout, " table file (eg /server/camra/star2ppm/star2ppm.table).\n");
		fprintf(stdout, "\n");
		fscanf(stdin, "%s", convertFname);
		fp = fopen(convertFname, "r");
	}

	num = 0;
	for (i=0; i<CODES; i++) {
		aaTable[i].atomNum = 0;
	}

	while (TRUE) {

		if (NULL == fgets(sVal, MAX_LINE_LEN, fp))
			break;

		nd = sscanf(sVal, "%s %d %s %s %s %s", res, &amb, old, n1, n2, n3);

		if (nd == 0) {
			fprintf(stdout, "found\n");
		}

		if ((nd >= 3) && (strlen(res) == 1)) {
			ix = res[0] - 'A';
			if ((ix >= 0) && (ix < CODES)) {
				j = aaTable[ix].atomNum;
				aaTable[ix].atom[j].amb = amb;
				strcpy(aaTable[ix].atom[j].oldAtom, old);
				if (nd == 6) {
					strcpy(aaTable[ix].atom[j].newAtom1, n1);
					strcpy(aaTable[ix].atom[j].newAtom2, n2);
					strcpy(aaTable[ix].atom[j].newAtom3, n3);
				} else if (nd == 5) {
					strcpy(aaTable[ix].atom[j].newAtom1, n1);
					strcpy(aaTable[ix].atom[j].newAtom2, n2);
					strcpy(aaTable[ix].atom[j].newAtom3, "");
				} else if (nd == 4) {
					strcpy(aaTable[ix].atom[j].newAtom1, n1);
					strcpy(aaTable[ix].atom[j].newAtom2, "");
					strcpy(aaTable[ix].atom[j].newAtom3, "");
				} else {
					strcpy(aaTable[ix].atom[j].newAtom1, "");
					strcpy(aaTable[ix].atom[j].newAtom2, "");
					strcpy(aaTable[ix].atom[j].newAtom3, "");
				}
				aaTable[ix].atomNum++;
				num++;
			}
		}
	}

	return(num);

}	/* read_convert */


/*************************************************************/
/* Read and process shifts from data file 
*/
static void proc_atoms(fp, outfp)
FILE *fp, *outfp;
{
	char res3[MAX_LINE_LEN], line[MAX_LINE_LEN];
	char sVal[MAX_LINE_LEN], *newAtom, atom[MAX_LINE_LEN];
	int i, j, nd, ch, id, ix, num, amb, stereo;
	double shift;

	num = 0;
	while (TRUE) {

        nd = fscanf(fp, "%d %d %s %s %*s %lf %*s %d",
			&line, &id, res3, sVal, &shift, &stereo);

		if (nd == 6) {
			upper_case(res3);
			ch = pres_3to1(res3);
			ix = ch - 'A';
			num = aaTable[ix].atomNum;

			if ((TRUE == valid_atom_name(sVal)) && (num > 0)) {
			for (i=0; i<num; i++) {
				if (strcmp(sVal, aaTable[ix].atom[i].oldAtom) == 0) {
					amb = aaTable[ix].atom[i].amb;
					if (strlen(aaTable[ix].atom[i].newAtom1) == 0) {
						fprintf(stdout, "Warning, cannot convert %s%d %s\n",
							res3, id, sVal);
					} else {
						newAtom = aaTable[ix].atom[i].newAtom1;
					 	sprintf(line, "1:%s_%d:%s         ", res3, id, newAtom);
						for (j=0; j<18; j++)
							putc(line[j], outfp);
						fprintf(outfp, "%7.2f\n", shift);
	
						if (strlen(aaTable[ix].atom[i].newAtom2) > 0) {
							newAtom = aaTable[ix].atom[i].newAtom2;
					 		sprintf(line, "1:%s_%d:%s       ", res3,id,newAtom);
							for (j=0; j<18; j++)
								putc(line[j], outfp);
							fprintf(outfp, "%7.2f\n", shift);
						} else {
							break;
						}
	
						if (strlen(aaTable[ix].atom[i].newAtom3) > 0) {
							newAtom = aaTable[ix].atom[i].newAtom3;
					 		sprintf(line, "1:%s_%d:%s       ", res3,id,newAtom);
							for (j=0; j<18; j++)
								putc(line[j], outfp);
							fprintf(outfp, "%7.2f\n", shift);
						}
					}
					break;
				}
			}
			if (i == num) {
				fprintf(stdout, "Error, cannot convert atom %s%d %s \n",
					res3, id, sVal);
			}
			}
		}
		if (NULL == fgets(sVal, MAX_LINE_LEN, fp))
			break;
	}

}	/* proc_atoms */


/*************************************************************/
/* Read shifts data file for atom labels
*/
static int valid_atom_name(name)
char *name;
{
	char ch;

	if (strlen(name) >= MAX_ATOM_NAME_LEN)
		return(FALSE);

    ch = name[0];
	if ((ch >= 'A') && (ch <= 'Z')) {
		return(TRUE);
	}

	return(FALSE);

}   /* valid_atom_name */

