
/*
	file: calc.c
	show all possible assignment of residues	
*/

#include "defs.h"
#include <math.h>

/* static vars */
static int chainNum;
static FILE *outfp;
static ASSIGN_REC assignChain[MAX_RULE_NUM];

/* procedure calls */
static void build_peakcon(), assign_peak();
static double test_peak();

extern void print_chain(), print_log_chain(), print_chain_summary();
extern void show_err(), print_run();
extern double fabs();


/*************************************************************/
/* find all connectivities 
*/
void do_peakcon()
{
	register int i, j;
	char sVal[MAX_LINE_LEN];

	fprintf(stdout, "\nStarting calculations...\n\n");

/* setup output file */
	if (NULL == (outfp = (FILE *)fopen(resultsFname, "w"))) {
		sprintf(sVal, "Cannot open data file: %s", resultsFname);
		show_err(TRUE, "do_peakcon", sVal);
	}
/*	print_run(outfp); */

/* main algorithm */
	chainNum = 0;
	for (i=0; i<ruleNum; i++) {
		for (j=0; j<atomNum; j++) {
			assignChain[i].pVal[j] = INIT_SHIFT;
		}
	}
	build_peakcon(0);
	print_chain_summary(stdout, chainNum);
	fprintf(stdout, "\nDone\n\n");

}	/* do_peakcon */


/*************************************************************/
/* build all possible residue peakcon
*/
static void build_peakcon(rNum)
int rNum;
{
	register int ix, j;
	int dim, numPeaks, matchCount, assignState;
	double prob;
	PEAK_REC *aPeak;

/* are we finished building the chain? */
	if (rNum == ruleNum) {
		print_chain(outfp, assignChain);
		/*
		print_log_chain(stdout, assignChain);
		*/
		chainNum++;
	}

	else {

		matchCount = 0;
		ix = connectRule[rNum].dataFileIndex;
		dim = dataFile[ix].nDim;
		assignState = connectRule[rNum].assignState;

/* process each peak as a candidate for adding to the chain */
		numPeaks = dataFile[ix].nData;
		for (j=0; j<numPeaks; j++) {
			aPeak = &dataFile[ix].peak[j];
			prob = test_peak(dim, rNum, aPeak);
			if (prob > 0) {
				matchCount++;
				if (assignState != 0) {
					assign_peak(rNum, dim, aPeak, prob);
					build_peakcon(rNum + 1);
					aPeak->assignNum--;
				}
			}
		}

/* continue building chain if we were looking for a break */
		if ((matchCount == 0) && (assignState == 0)) {
			assignChain[rNum].IDnum = 0; 
			assignChain[rNum].prob = 1.0;
			build_peakcon(rNum + 1);
		}
	}

}	/* build_peakcon */


/*************************************************************/
/* test if a specific peak can be added to our current chain 
*/
static double test_peak(dim, num, aPeak)
register int dim, num;
PEAK_REC *aPeak;
{
	register int i, j, ix;
	double coord, cErr, cVal, aProb, lowProb;

/* if this peak has been assigned, then outta here */
	if (aPeak->assignNum > 0)
		return(0.0);

	lowProb = 1.0;
	for (i=0; i<dim; i++) {
		coord = aPeak->coord[i];

/* check if there are any boundary constraints from relation data */
		if (coord < connectRule[num].relTerm[i].lb)
			return(0.0);
		if (coord > connectRule[num].relTerm[i].ub)
			return(0.0);

		ix = connectRule[num].relTerm[i].atomIx;
		for (j=0; j<num; j++) {
			cVal = assignChain[j].pVal[ix];
			if (connectRule[j].dataFileIndex == connectRule[num].dataFileIndex)
				cErr = dataFile[connectRule[num].dataFileIndex].fferr[i];
			else
				cErr = dataFile[connectRule[num].dataFileIndex].f2ferr[i];

/* compare each peak coord to our chain's current values */
			if (cVal != INIT_SHIFT) {
				aProb = 1.0 - (fabs(cVal - coord) / cErr);
				if (aProb <= 0.0)
					return(0.0);
				else {
					lowProb = MIN(aProb, lowProb);
				}
			}
		}
	}

	return(lowProb);

}	/* test_peak */


/*************************************************************/
/* update data structures when assigning a peak to the chain 
*/
static void assign_peak(num, dim, aPeak, prob)
int num, dim;
PEAK_REC *aPeak;
double prob;
{
	register int i, j, ix;

/* record the coordinates of the peak we are adding */
	for (i=0; i<dim; i++) {
		ix = connectRule[num].relTerm[i].atomIx;
		assignChain[num].pVal[ix]= aPeak->coord[i];
	}

/* record the peak Id */
	assignChain[num].IDnum = aPeak->IDnum;

/* how well the peak fits the rules */
	assignChain[num].prob = prob;

/* indicate that this peak is now assigned */
	aPeak->assignNum++;

}	/* assign_peak */
