
/*
	file: data.c
	read peak pick files for peakcon.	
*/

#include "defs.h"

/* local variables */

static int nData;

/* procedure calls */

static void read_data();
static int read_peaks();
extern void show_err();
extern int atoi();


/*************************************************************/
/* reads all the data files. 
*/
void read_data_files()
{
	register int i;

	fprintf(stdout, "\nReading data files...\n");
	for (i=0; i<dataFileNum; i++) {
		read_data(&dataFile[i]);
		fprintf(stdout, " Peaks Read: %d  File: %s\n", 
			dataFile[i].nData, dataFile[i].fName);
	}

} /* read_data_files */


/*************************************************************/
/* reads one data input file. 
*/
static void read_data(dRec)
DATA_FILE_REC *dRec;
{
	FILE *fp;
	char *rc, sVal[MAX_LINE_LEN];

/* open data file */
	fp = fopen(dRec->fName, "r");
	if (fp == NULL) {
		sprintf(sVal, "Could not open data file %s", dRec->fName);
		show_err(TRUE, "read_data", sVal);
	}

/* parse each line */

	rc = fgets(sVal, MAX_LINE_LEN-1, fp);
	nData = 0;
	while (rc != NULL) {
		if (TRUE == read_peaks(sVal, dRec))
			nData++;
		rc = fgets(sVal, MAX_LINE_LEN-1, fp);
	}

	fclose(fp);
	dRec->nData = nData;

}	/* read_data */


/*************************************************************/
/* parse one line from a data input file. 
*/
static int read_peaks(line, dRec)
char *line;
DATA_FILE_REC *dRec;
{
	register int i, nd, nField, nFound;
	char *s1, sVal[MAX_LINE_LEN];

/* make sure the line is not commented out */
	s1 = strtok(line, " \t");
	if (s1[0] == COMMENT) 
		return(FALSE);

/* make sure we can handle the data */
	if (nData >= MAX_DATA_NUM) {
		sprintf(sVal, "Max data file size exceeded: Max = %d\n", nData);
		show_err(TRUE, "read_peaks", sVal);
	}

/* read all the fields */
	nField = 1;
	nFound = 0;
	dRec->peak[nData].IDnum = nData;
	while (s1 != NULL) {
		if (nField == dRec->labelIx) {
			dRec->peak[nData].IDnum = atoi(s1);
		}
		for (i=0; i<dRec->nDim; i++) {
			if (nField == dRec->dataIx[i]) {
				nd = sscanf(s1, "%lf", &dRec->peak[nData].coord[i]);
				if (nd == 0) {
					return(FALSE);
				} else {
					nFound++;
				}
			}
		}
		s1 = strtok((char *)NULL, " \t");
		nField++;
	}

	if (nFound != dRec->nDim)
		return(FALSE);

	dRec->peak[nData].assignNum = 0;
	return(TRUE);

}	/* read_peaks */
