
/*
	file...: main.c
	purpose: generalized connectivities program 

	author.: Robert Boyko
	date...: April 27, 1999	
	place..: University of Alberta / Department of Biochemistry
*/

#include "defs.h"

/* local variables */
static FILE *rulesfp;

/* global vars */
char rulesFname[MAX_FNAME_LEN], resultsFname[MAX_FNAME_LEN];
int dataFileNum;

/* procedure calls */
static void read_args();
extern void do_peakcon(), read_data_files();
extern void read_rules_file(), show_err(), print_run();


/*************************************************************/
/* main program
*/
int main(argc, argv)
int argc;
char *argv[];
{

	read_args(argc, argv);
	read_rules_file(rulesfp);
	read_data_files();
	do_peakcon();
	return(0);

}	/* main */


/*************************************************************/
/* Read program arguments, io files 
*/
static void read_args(argc, argv)
int argc; 
char *argv[];
{
	char sVal[MAX_LINE_LEN];
	int i;
		 
	print_run(stdout);

	if (argc < 3) {
		sprintf(sVal, "Usage: peakcon rulesFile [ dataFiles ] resultsFile");
		show_err(TRUE, "read_args", sVal);
	}

/* read and open rules file */
 
	strcpy(rulesFname, argv[1]);
	if (NULL == (rulesfp = (FILE *)fopen(rulesFname, "r"))) {
		 sprintf(sVal, "Cannot open rules file: %s", rulesFname);
		 show_err(TRUE, "read_args", sVal);
	}
	fprintf(stdout, "\nRules file: %s\n", rulesFname);

/* read optional data files */

	for (i=2; i<argc-1; i++) {
		strcpy(dataFile[i-2].fName, argv[i]);
		if (NULL == (FILE *)fopen(dataFile[i-2].fName, "r")) {
			sprintf(sVal, "Cannot open data file %d: %s", i-1, argv[i]);
			show_err(TRUE, "read_args", sVal);
		}
		fprintf(stdout, "Data file %d: %s\n", i-1, dataFile[i-2].fName);
	}
	dataFileNum = argc - 3;

/* get output file name */
 
	strcpy(resultsFname, argv[argc-1]);
	fprintf(stdout, "Output file: %s\n", resultsFname);

}	/* read_args */
