
/*
	file: print.c
	This file contains routines for printing results. 
*/

#include "defs.h"

/* procedure calls */
static double find_ave_shift(), find_prob();
extern char *get_date();

/*************************************************************/
/* Print info about this run. 
*/
void print_run(fp)
FILE *fp;
{

	fprintf(fp, "#\n");
	fprintf(fp, "#*****************************************************\n");
	fprintf(fp, "#\n");
	fprintf(fp, "#  Program......: peakcon (%s)\n", VERSION);
	fprintf(fp, "#  Description..: Generalized Connectivity Program \n");
	fprintf(fp, "#  Date.........: %s", get_date());
	fprintf(fp, "#\n");
	fprintf(fp, "#*****************************************************\n");
	fprintf(fp, "#\n");
	fflush(fp);

}	/* print_run */


/*************************************************************/
/* Print one chain matched. 
*/
void print_chain(fp, resChain)
FILE *fp;
ASSIGN_REC *resChain;
{
	register int i, ruleID;
	double shift, prob, fabs();
	char *pFormat, *pix, sForm[MAX_LINE_LEN];

	for (i=0; i<printNum; i++) {

		pFormat = printData[i].printFormat;

/* print shift value */
		if (printData[i].pType == P_MATCH) {
			shift = find_ave_shift(resChain, printData[i].atomIx);
			if (strchr(pFormat, 'd') != NULL) {
				fprintf(fp, pFormat, (int)shift);
			} else {
				if (strchr(pFormat, 'a') != NULL) {
					strcpy(sForm, pFormat);
					pix = strchr(sForm, 'a');
					*pix = 'f';
					fprintf(fp, sForm, fabs(shift));
				} else {
					fprintf(fp, pFormat, shift);
				}
			}
		} else {

/* print IDnum */
			if (printData[i].pType == P_RULEID) {
				ruleID = printData[i].ruleID;
				fprintf(fp, pFormat, resChain[ruleID-1].IDnum);
			} else {

/* print prob fit score */
				if (printData[i].pType == P_STRING) {
					fprintf(fp, pFormat, printData[i].printBuf);
				} else {

/* print an arbitrary string */
					if (printData[i].pType == P_PROB) {
						prob = find_prob(resChain);
						fprintf(fp, pFormat, prob);
					}
				}
			}
		}
		fprintf(fp, " ");
	}

	fprintf(fp, "\n");
	fflush(fp);

}	/* print_chain */


/*************************************************************/
/* Print one chain matched. 
*/
void print_log_chain(fp, resChain)
FILE *fp;
ASSIGN_REC *resChain;
{
	register int i, j, ix;

	for (i=0; i<ruleNum; i++) {
		fprintf(fp, " %5d", resChain[i].IDnum);
		for (j=0; j<4; j++) {
			ix = connectRule[i].relTerm[j].atomIx;
			fprintf(fp, " %6.3f", resChain[i].pVal[ix]);
		}
		fprintf(fp, " %5.3f", resChain[i].prob);
		fprintf(fp, "\n");
	}
	fprintf(fp, "\n");
	fflush(fp);

}	/* print_log_chain */


/*************************************************************/
/* Print chain stats for one relation 
*/
void print_chain_summary(fp, total)
FILE *fp;
int total;
{

	fprintf(fp, "#\n");
	fprintf(fp, "####### Number of chains found: %d #######\n", total);
	fprintf(fp, "#\n");
	fflush(fp);

}	/* print_chain_summary */


/*************************************************************/
/* find average shift values from chained data 
*/
static double find_ave_shift(resChain, ix)
ASSIGN_REC *resChain;
int ix;
{
	register int j, cNum;
	double cTot, cVal;

	cTot = 0.0;
	cNum = 0;
	for (j=0; j<ruleNum; j++) {
		cVal = resChain[j].pVal[ix];
		if (cVal != INIT_SHIFT) {
			cTot += cVal;
			cNum++;
		}
	}
	if (cNum == 0)
		return(INIT_SHIFT);

	return(cTot / (double)cNum);

}	/* find_ave_shift */


/*************************************************************/
/* find overall probability from chained data 
*/
static double find_prob(resChain)
ASSIGN_REC *resChain;
{
	register int j;
	double currProb;

	currProb = 1.0;
	for (j=0; j<ruleNum; j++) {
		currProb = MIN(resChain[j].prob, currProb);
	}

	return(currProb);

}	/* find_prob */
