
######################################################################
#
#  events for handling assignment/unassignment of chains to sequence 
#
######################################################################

######################################################################
#
# user clicks on the assign button 
#
proc assign_add_item { } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set aaIx $snbVar(assignIx)
	set prevLabel $myChain($cur,cLabel)

	# update data structures for the given chain assignment
	if { [assign_add_update $cur $aaIx] < 0 } { return }

	# update files and screen 
	event_assign_update

	usnb_log file "Assigned $prevLabel to $myChain($cur,cLabel)"
}

######################################################################
#
# user clicks on the un-assign button 
#
proc assign_delete_item { } {
	global snbVar snbConst myChain

	set cur $myChain(current)
	set prevLabel $myChain($cur,cLabel)

	# update sequence data structure
	if { [assign_delete_update $cur] < 0 } { return }

	# make sure everything else is up to date 
	event_assign_update

	usnb_log file "Un-Assign button: Unassigned $prevLabel"
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# assign button update event 
#
proc event_assign_update { } {
	global snbVar snbConst mySpectra

	# an assignment event changes the chains, the peaklists, and ppm.out
	data_write_chains $snbVar(fn,chains)
	data_write_ppm $snbVar(fn,outppm)
	data_write_xpk $mySpectra(xpk,outFnames)
	usnb_log file "Updated xpk files: $mySpectra(xpk,outFnames)"

	# update the nmrview display
	nvdraw_update_peaklists $snbVar(parm,readpks)

	# update menubar and sequence display 
	panel_menubar_chains
	sequence_panel_update "FALSE"
}

######################################################################
#
# assign shifts in the current chain to particular part of the sequence
#
proc assign_add_update { cur aaIx } {
	global snbVar snbConst myChain mySeq mySpectra myConnect

	# make sure that this assignment works here
	if { [check_assign $cur $aaIx] < 0 } { return -1 }

	# record where in the sequence this chain is assigned
	set myChain($cur,aaIx) $aaIx

	# record each specific shift and update sequence
	set numConnect $myChain($cur,nConnect)
	for {set i 1} {$i <= $numConnect} {incr i} {
		set cix $myChain($cur,$i,cix)
		set pix $myChain($cur,$i,pix)
		set aIndex 0
		if { $myConnect($cix,$pix,cStatus) == $snbConst(NO_VAL) && $i > 1 } {
			set j [expr $i - 1 ]
			set cix $myChain($cur,$j,cix)
			set pix $myChain($cur,$j,pix)
			set aIndex 1
		}
		foreach atomName $mySpectra(atoms,names) {
			set shift0 [ connect_get_val $cix $pix $mySpectra(connect,$atomName,$aIndex) ]
			set mySeq($aaIx,$atomName,snbAssign) $shift0
		}

		set mySeq($aaIx,assignChainIx) $cur
		incr aaIx 
	}

	# same as above for the last entry in the chain (atom 1)
	set cix $myChain($cur,$numConnect,cix)
	set pix $myChain($cur,$numConnect,pix)
	foreach atomName $mySpectra(atoms,names) {
		set shift1 [ connect_get_val $cix $pix $mySpectra(connect,$atomName,1) ]
		set mySeq($aaIx,$atomName,snbAssign) $shift1
	}
	set mySeq($aaIx,assignChainIx) $cur

	# update xpk files
	xpk_assign_add $cur $myChain($cur,aaIx)

	# make new label for the assignment
	chains_make_assign_label $cur

	return 0
} 

######################################################################
#
# some checks before we honor user's request for assignment 
#
proc check_assign { cur aaIx } {
	global snbVar snbConst myChain mySeq

	# do nothing if chain is length 0. 
	set numConnect $myChain($cur,nConnect)
	if { $numConnect == 0 } {
		usnb_err "Cannot assign a chain of length 0"
		return -1
	}

	# see if any of the amino acids that we want to assign are already assigned
	for {set i 0} {$i <= $numConnect} {incr i} {
		set acx $mySeq($aaIx,assignChainIx)
		if { $acx >= 0 && $acx != $cur } {
			set aaNum $mySeq($aaIx,aaNum)
			set aLabel $myChain($acx,cLabel)
			usnb_err "Residue $aaNum has already been assigned to $aLabel"
			return -1
		}
		incr aaIx 
	}
	return 0
}

######################################################################
#
# assign current chain individual shifts to the peaklists 
#
proc xpk_assign_add { cur aaIx } {
	global snbVar snbConst myChain mySeq mySpectra

	set numConnect $myChain($cur,nConnect)
	for {set i 1} {$i <= $numConnect} {incr i} {
		set aaNum $mySeq($aaIx,aaNum)
		set cix $myChain($cur,$i,cix)
		set pix $myChain($cur,$i,pix)
		xpk_assign_add_update $cix $pix $aaNum $mySpectra(connect,RES,0)
		incr aaIx
	}
	set aaNum $mySeq($aaIx,aaNum)
	set cix $myChain($cur,$numConnect,cix)
	set pix $myChain($cur,$numConnect,pix)
	xpk_assign_add_update $cix $pix $aaNum $mySpectra(connect,RES,1)
}

######################################################################
#
# assign current chain individual shifts to the peaklists 
#
proc xpk_assign_add_update { cix pix aaNum peakIds } {
	global snbVar snbConst myPeak myConnect mySpectra

	# process each xpk file 
	foreach i $peakIds {
		set peakId $myConnect($cix,$pix,$i)
		if { $peakId < 0 } {
			continue
		}
		set xpkix $mySpectra(connect,XPK_INDEX,$i)
		set iPeak $myPeak($xpkix,index,$peakId)
		if [info exists myPeak($xpkix,foldedLine,$iPeak) ] {
			set line $myPeak($xpkix,foldedLine,$iPeak)
		} else {
			set line $myPeak($xpkix,origLine,$iPeak)
		}

		# process each field in one xpk peak record 
		set dim [expr [llength $mySpectra(xpk,fields,$xpkix)] - 1]
		for {set j 0} {$j < $dim } {incr j} {

			# get the replacement string and field
			set atomString [lindex $mySpectra(connect,ANAME,$i) $j]
			set atomInfo [split $atomString :]
			set atomName [lindex $atomInfo 0]
			set atomName [string tolower $atomName]
			set newResId [expr $aaNum + [lindex $atomInfo 1]]
			set assignName "{$newResId.$atomName}"
			set field [expr [lindex $mySpectra(xpk,fields,$xpkix) $j] - 2 ]

			# replace the old string with the new, treat string as a list
			# formatting will be lost but not sure how else to handle this
			set line [lreplace $line $field $field $assignName]
		}

		# remove extra braces and '?' will have braces missing for nmrview
		regsub -all "{{" $line "{" line
		regsub -all "}}" $line "}" line
		regsub -all {\?} $line \{&\} line

		if ![info exists myPeak($xpkix,assignLine,$iPeak) ] {
			set myPeak($xpkix,assignLine,$iPeak) $line
		}
	}
}

######################################################################
#
# un-assign shifts of particular part of the sequence
#
proc assign_delete_update { cur } {
	global snbVar snbConst myChain mySeq mySpectra

	# make sure that un-assignment works here
	if { $myChain($cur,aaIx) < 0 } {
		usnb_err "The current chain is already un-assigned"
		return -1
	}

	# un-assign this chain from each amino acid in our sequence 
	set numConnect $myChain($cur,nConnect)
	set aaIx $myChain($cur,aaIx)
	for {set i 0} {$i <= $numConnect} {incr i} {
		if { $mySeq($aaIx,assignChainIx) == $cur } {
			set mySeq($aaIx,assignChainIx) -1
		}
		foreach atomName $mySpectra(atoms,names) {
			unset mySeq($aaIx,$atomName,snbAssign)
		}
		incr aaIx
	}

	# update xpk files
	xpk_assign_delete $cur $myChain($cur,aaIx)

	# finally set the chain to un-assigned mode and update its label
	set myChain($cur,aaIx) -1
	chains_make_unassign_label $cur

	return 0
}

######################################################################
#
# un-assign shifts (of one chain) from our peaklists 
#
proc xpk_assign_delete { cur aaIx } {
	global snbVar snbConst myChain mySeq mySpectra

	set numConnect $myChain($cur,nConnect)
	for {set i 1} {$i <= $numConnect} {incr i} {
		set cix $myChain($cur,$i,cix)
		set pix $myChain($cur,$i,pix)
		xpk_assign_delete_update $cix $pix $mySpectra(connect,RES,0)
		incr aaIx
	}
	set cix $myChain($cur,$numConnect,cix)
	set pix $myChain($cur,$numConnect,pix)
	xpk_assign_delete_update $cix $pix $mySpectra(connect,RES,1)
}

######################################################################
#
# un-assign shifts (of one peak) from our peaklists 
#
proc xpk_assign_delete_update { cix pix peakIds } {
	global snbVar snbConst myPeak myConnect mySpectra

	# process each peak pick file 
	foreach i $peakIds {
		set peakId $myConnect($cix,$pix,$i)
		if { $peakId < 0 } {
			continue
		}
		set xpkix $mySpectra(connect,XPK_INDEX,$i)
		set iPeak $myPeak($xpkix,index,$peakId)
		if [info exists myPeak($xpkix,assignLine,$iPeak) ] {
			unset myPeak($xpkix,assignLine,$iPeak)
		}
	}
}
