######################################################################
#
# initialization procedure for snb
#
######################################################################

######################################################################
#
# create main initialization window 
#
proc init_snb_win { } {
	global snbVar snbConst

	set snbVar(OKFLAG) 1
	set f [usnb_show_win snbinit "Smartnotebook Initialization Procedure"]
	if { $f == "FALSE" } { return }

	# read the experiment sets snb supports 
	read_exp_info

	# try and find a default web browser 
	usnb_find_browser

	# start with first frame
	frame_snb_1 $f.main
	tkwait window $f 

	# write init file and exit ok if all went well
	if { $snbVar(OKFLAG) == 0 } {
		init_snb_write $snbVar(fn,snbInit,user)
		return 0
	}

	return 1
}

######################################################################
#
# Init Screen 1 - get snb output directory and default settings
#
proc frame_snb_1 { f } {
	global snbVar snbConst

	pack [frame $f] -fill x -expand true -anchor n -padx 20
    label $f.titleLabel -text "Welcome to smartnotebook - Initialization (1 of 6)"
    pack $f.titleLabel

	pack [frame $f.1] -fill x -expand true
	message $f.1.intro -text \
		"Smartnotebook writes ALL output files to the directory specified\
		below. Files\
		which are created include assigned peaklists, current chains list,\
		connectivity lists, and folded peak lists."
	pack $f.1.intro

    u_file_entry $f.1.input    "Output directory " \
		18 snbVar(fn,session) 44 $snbVar(snbwin,padx) { } 0 0
    pack $f.1.input -pady 15

	pack [frame $f.2] -fill x -expand true
	message $f.2.intro -text \
		"All interactive help files are written in html. Please\
		indicate the location of your web browser in order to view\
		these files."
	pack $f.2.intro

    u_file_entry $f.2.browser    "Web browser " \
		18 snbVar(fn,browser) 44 $snbVar(snbwin,padx) { } 0 0
    pack $f.2.browser -pady 15

	# button widgets
	pack [frame $f.buttons] -pady 15
	pack [button $f.buttons.save -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_snb_outdir $f"] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "abort_snb_init"] -side left
}

######################################################################
#
# check snb output directory 
#
proc do_snb_outdir { f } {
	global snbVar snbConst

	if [file isfile $snbVar(fn,session)] {
		usnb_err "' $snbVar(fn,session)' already exists as a file."
	}

	if ![file executable $snbVar(fn,browser)] {
		usnb_err "' $snbVar(fn,browser)' is not an executable file."
	}

	# create output directory if it doesn't exist already
	if ![file isdirectory $snbVar(fn,session)] {
		sys_make_dir $snbVar(fn,session)
	}

	# Define standard experiment independent file names which
	# we use in the output directory
	set snbVar(fn,version) $snbVar(fn,session)/VERSION
	set snbVar(fn,outppm) $snbVar(fn,session)/ppm.out
	set snbVar(fn,chains) $snbVar(fn,session)/chains
	set snbVar(fn,suggest) $snbVar(fn,session)/chains.suggest

	usnb_log all "Output directory: $snbVar(fn,session)"
	usnb_check_version

	# go on to next window
	usnb_kill_win_nosave $f
	frame_snb_2 $f
}

######################################################################
#
# Init Screen 2 - Gather info about the type of experiment 
#
proc frame_snb_2 { f } {
	global snbVar snbConst mySpectra

	set ySpace 5

	# Important! Forget about any previous info about current experiment
	if [info exists mySpectra] { 
		unset mySpectra
	}
	
	set mySpectra(exp,spec) 1
	set mySpectra(exp,type) [lindex $snbVar(exp,types) 0] 
	set w [lindex $snbVar(exp,types) 0]
	set mySpectra(exp,ref) [string trim [lindex [lindex $snbVar(exp,$w) 0] 0]]

	pack [frame $f] -fill x -expand true -anchor n -padx 20
    label $f.titleLabel -text "Initialize Experiment (2 of 6)"
    pack $f.titleLabel

	message $f.intro -text \
		"Please select from the following experiment types and\
		reference experiments that are currently defined in smartnotebook.\
		Keep clicking the NEXT button until you have decided upon a spectral \
		strips window configuration. Finally press CONTINUE."
	pack $f.intro 

	# experiment types widget
	pack [frame $f.type]
	label $f.type.boldLabel -text "Experiment Type:" -width 25 -anchor w
	pack $f.type.boldLabel -side left -pady 5
	foreach t $snbVar(exp,types) {
		radiobutton $f.type.$t -value $t -text "$t  " \
			-variable mySpectra(exp,type) \
			-command "do_spectral_choices $f 0"
		pack $f.type.$t -side left -padx 5
	}

	# reference widget 
	pack [frame $f.ref]
	label $f.ref.boldLabel -text "Reference Experiment:" -width 25 -anchor w
	pack $f.ref.boldLabel -side left 
	foreach w $snbVar(exp,types) {
		foreach t $snbVar(exp,$w) {
			set r [string trim [lindex $t 0]]
			if ![winfo exists $f.ref.$r] {
				radiobutton $f.ref.$r -value $r -text "$r  " -state disabled \
					-variable mySpectra(exp,ref) \
					-command "do_spectral_choices $f 0"
				pack $f.ref.$r -side left -padx 5
			}
		}
	}

	# draw spectral configuration widget
	frame $f.spec -relief raised -bd 2
	pack $f.spec -side top -expand true -pady 10 -ipady $ySpace -ipadx 10
	frame $f.spec.fr1
	pack $f.spec.fr1 -pady 10
	frame $f.spec.fr2
	pack $f.spec.fr2
	label $f.spec.fr1.boldLabel -text "Spectral Configuration:"
	pack $f.spec.fr1.boldLabel -side left 
	label $f.spec.fr1.lab2
	pack $f.spec.fr1.lab2 -side left -padx 5 
	pack [frame $f.spec.fr1.buttons] -padx 5
	button $f.spec.fr1.buttons.next -text Next -pady $ySpace \
		-padx $snbVar(snbwin,padx) \
		-command "do_spectral_choices $f 1"
	pack $f.spec.fr1.buttons.next -side left
	for {set i 0} {$i < $snbVar(snbinit,maxSpec)} {incr i} {
		pack [label $f.spec.fr2.$i -anchor w -width 30 ] -anchor w
	}

	# button widgets
	pack [frame $f.buttons] -pady 15
	pack [button $f.buttons.save -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_snb_exptype $f" ] -side left
	pack [button $f.buttons.back -text "Back" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_nosave $f; frame_snb_1 $f" ] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "abort_snb_init"] -side left

	do_spectral_choices $f 0
}

######################################################################
#
# read the EXPERIMENTS file to determine the reference types
# search for the line REF: in the experiments file.
# since we're here, also read the experiment sets.
#
proc read_exp_info { } {
	global snbVar snbConst

	set snbVar(exp,types) ""

	if  {[ catch { set fp [open $snbVar(fn,expList) r ] } err]}  {
		usnb_err "Cannot read experiment definition file $snbVar(fn,expList)"
	}

	while {[gets $fp line] >= 0} {

		set ret [regexp -nocase {^EXP: (.*)} $line dummy expType]
		if { $ret == 1 } {
			set expType [ string trim $expType ]
			lappend snbVar(exp,types) $expType
			continue
		} 

		set ret [regexp -nocase {^REF: (.*)} $line dummy refType]
		if { $ret == 1 } {
			set refType [ string trim $refType ]
			lappend snbVar(exp,$expType) $refType
			set snbVar(exp,$expType,$refType,num) 0
			continue
		}

		set ret [regexp -nocase {^#} $line ]
		if { $ret != 1} { 
			set ref [ string trim [lindex $line 0] ]
			lappend snbVar(exp,$expType,$refType) $line
			incr snbVar(exp,$expType,$refType,num)
		}
	}
	close $fp
}

######################################################################
#
# update variables and spectral display choices based on
# experiment choices.
#
proc do_spectral_choices { f nextSpec } {
	global snbVar snbConst mySpectra

	set eId $mySpectra(exp,type)
	set rId $mySpectra(exp,ref)

	set mySpectra(exp,spec) [expr $mySpectra(exp,spec) + $nextSpec];
	if { $mySpectra(exp,spec) > $snbVar(exp,$eId,$rId,num) } {
		set mySpectra(exp,spec) 1
	}

	foreach t $snbVar(exp,$eId) {
		set r [string trim [lindex $t 0]]
		$f.ref.$r configure -state normal
	}

	set header "$mySpectra(exp,spec) of $snbVar(exp,$eId,$rId,num)"
	$f.spec.fr1.lab2 configure -text $header

	set ix [expr $mySpectra(exp,spec) - 1]
	set line [lindex $snbVar(exp,$eId,$rId) $ix]
	set count 0
	foreach t $line {
		# skip first field which is not needed for display 
		if { $count > 0 } {
			$f.spec.fr2.$count configure -text "Strip $count: $t"
		}
		incr count
	}
	for {set i $count} {$i < $snbVar(snbinit,maxSpec)} {incr i} {
		$f.spec.fr2.$i configure -text ""
	}
}

######################################################################
#
# update users snb configuration file 
#
proc do_snb_exptype { f } {
	global snbVar snbConst mySpectra 

	set confDir $snbVar(fn,expdir)/$mySpectra(exp,type)
	set confFile $mySpectra(exp,ref)-$mySpectra(exp,spec)

	# here are the key variables which define our experiment
	set mySpectra(fn,expInit,lib) 	$confDir/$confFile
	set mySpectra(fn,expInit,user) 	$snbVar(fn,session)/snb.$confFile
	set mySpectra(fn,nvInit,lib) 	$confDir/$confFile.nmrview
	set mySpectra(fn,nvInit,user)	$snbVar(fn,session)/snb.$confFile.nmrview

	# rules directory
	set mySpectra(fn,rulesdir) 		$confDir/$confFile.rules
	if ![file exists $mySpectra(fn,rulesdir)] {
		set mySpectra(fn,rulesdir) 		$confDir/$mySpectra(exp,ref).rules
	}

	# search rules
	set mySpectra(fn,searchItems)	$confDir/$confFile.search
	if ![file exists $mySpectra(fn,searchItems)] {
		set mySpectra(fn,searchItems) 	$confDir/$mySpectra(exp,ref).search
	}

	# get default settings for this experiment type
	if ![file exists $mySpectra(fn,expInit,lib)] {
		usnb_err "Configuration file not found: ' $mySpectra(fn,expInit,lib)'"
	}
	source $mySpectra(fn,expInit,lib)
	usnb_log all "Experiment: $mySpectra(exp,type)"
	usnb_log all "Reference: $mySpectra(exp,ref)"
	usnb_log all "Spectral Display: $mySpectra(exp,spec)"
	usnb_log all "Sourcing: $mySpectra(fn,expInit,lib)"

	# go on to next window
	usnb_kill_win_nosave $f
	frame_snb_3 $f
}

######################################################################
#
# Init Screen 3 - Read Spectral data files 
#
proc frame_snb_3 { f } {
	global snbVar snbConst mySpectra

	if [info exists mySpectra(hsqc,winName)] {
		unset mySpectra(hsqc,winName)
	}
	if [info exists mySpectra(hsqc,nv,0)] {
		unset mySpectra(hsqc,nv,0)
	}

	pack [frame $f] -fill x -expand true -anchor n -padx 20
    label $f.titleLabel -text "Initialize Spectral Data Files (3 of 6)"
    pack $f.titleLabel

	message $f.intro -text \
		"Indicate where smartnotebook can find the peaklists and processed\
		spectra files. Input peaklists can be in nmrDraw or nmrview format.\
		You can use the BROWSE key to navigate through\
		your files."
	pack $f.intro 

	# peaklist file names
	frame $f.xpk
	pack $f.xpk -pady 15 -fill both -expand true
	label $f.xpk.boldLabel -text "Nmrview/NmrDraw PeakLists"
	pack $f.xpk.boldLabel 

	for {set i 0} {$i < $mySpectra(xpk,fileNum)} {incr i} {
		if [info exists mySpectra(xpk,file,$i)] {
			unset mySpectra(xpk,file,$i)
		}
		if ![file exists $mySpectra(xpk,defName,$i)] {
			set mySpectra(xpk,defName,$i) ""
		}
		u_file_entry $f.xpk.$i "$mySpectra(xpk,desc,$i)" \
			18 mySpectra(xpk,defName,$i) 32 $snbVar(snbwin,padx) { } 1 1
		pack $f.xpk.$i
	}

	# processed spectra file names
	frame $f.nv
	pack $f.nv -pady 15 -fill both -expand true
	label $f.nv.boldLabel -text "Processed NMR Data"
	pack $f.nv.boldLabel 
	for {set i 0} {$i < $mySpectra(nv,fileNum)} {incr i} {
		if [info exists mySpectra(nv,file,$i)] {
			unset mySpectra(nv,file,$i)
		}
		if ![file exists $mySpectra(nv,defName,$i)] {
			set mySpectra(nv,defName,$i) "" 
		}
		u_file_entry $f.nv.$i "$mySpectra(nv,desc,$i)" \
			18 mySpectra(nv,defName,$i) 32 $snbVar(snbwin,padx) { } 1 0
		pack $f.nv.$i
	}

	# par files (optional)
	frame $f.par
	pack $f.par -pady 15 -fill both -expand true
	label $f.par.boldLabel -text "Nmrview par files (optional)"
	pack $f.par.boldLabel 
	for {set i 0} {$i < $mySpectra(nv,fileNum)} {incr i} {
		if [info exists mySpectra(par,file,$i)] {
			unset mySpectra(par,file,$i)
		}
		set mySpectra(par,defName,$i) "" 
		if [file exists $mySpectra(nv,defName,$i)] {
			set root [u_file_prefix $mySpectra(nv,defName,$i) "."]
			set pfn "$root.par"
			if [file exists $pfn] {
				set mySpectra(par,defName,$i) $pfn
			}
		}
		u_file_entry $f.par.$i "$mySpectra(nv,desc,$i)" \
			18 mySpectra(par,defName,$i) 32 $snbVar(snbwin,padx) { } 1 0
		pack $f.par.$i
	}

	# button widgets
	pack [frame $f.buttons] -pady 15
	pack [button $f.buttons.save -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_snb_expdata $f"] -side left
	pack [button $f.buttons.back -text "Back" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_nosave $f; frame_snb_2 $f" ] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "abort_snb_init"] -side left
}

######################################################################
#
# User hits CONTINUE button of previous window
#
proc do_snb_expdata { f } {
	global snbVar snbConst

	usnb_check_spectra defName
	usnb_check_par defName
	# go on to next window
	usnb_kill_win_nosave $f
	frame_snb_4 $f
}

######################################################################
#
# Init Screen 4 - How do we get shift constraints files
#
proc frame_snb_4 { f } {
	global snbVar snbConst

	pack [frame $f] -fill x -expand true -anchor n -padx 20
    label $f.titleLabel -text "Calculating Shift Constraints (4 of 6)"
    pack $f.titleLabel

	message $f.intro -text \
		"Select the radio button of your preferred choice and enter\
		any corresponding input files. If you are unfamiliar with the\
		importance of the chemical shifts constraints file, please\
		select the HELP button first."
	pack $f.intro 

	pack [frame $f.fr] -pady 15
	radiobutton $f.fr.okRadio -variable snbVar(snbinit,constraints) \
		-text "I have my own shift constraints file    " \
		-value "ok" -command "init_cinp_select $f"
	pack $f.fr.okRadio -padx 5 -pady 5 -anchor w

	radiobutton $f.fr.def -value "def" -variable snbVar(snbinit,constraints) \
		-text "Generate shift constraints from nmrview sequence file using BMRB stats  " \
		-command "init_cinp_select $f"
	pack $f.fr.def -padx 5 -pady 5 -anchor w

	radiobutton $f.fr.orbf -value "orb" -variable snbVar(snbinit,constraints) \
		-text "Generate shift constraints from homologous sequence shift files...  " \
		-command "init_cinp_select $f"
	pack $f.fr.orbf -padx 5 -pady 5 -anchor w

	pack [frame $f.file] -pady 15 

	# button widgets
	pack [frame $f.buttons] -pady 15
	pack [button $f.buttons.exe -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_snb_cinp $f"] -side left
	pack [button $f.buttons.back -text "Back" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_nosave $f; frame_snb_3 $f" ] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "abort_snb_init"] -side left
	pack [button $f.buttons.help -text "Help" \
		-padx $snbVar(snbwin,padx) \
		-command "sys_run_browser $snbVar(help,seq)"] -side left

	set snbVar(snbinit,constraints) "def"
	$f.fr.def invoke
}

######################################################################
#
# Change fields based on radio button selected
#
proc init_cinp_select { f } {
	global snbVar snbConst

	destroy $f.file.ent
	if  {$snbVar(snbinit,constraints) == "def"} {
		u_file_entry $f.file.ent "Sequence File" \
			18 snbVar(fn,seq) 36 $snbVar(snbwin,padx) { } 1 1
	}
}

######################################################################
#
# Decide how we create the shift constraints file 
#
proc do_snb_cinp { f } {
	global snbVar snbConst

	if { $snbVar(snbinit,constraints) == "ok" } {
		usnb_kill_win_nosave $f
		frame_snb_5 $f
	}

	if { $snbVar(snbinit,constraints) == "def" } {
		if ![file readable $snbVar(fn,seq)] {
			usnb_err "Cannot find nmrview sequence file ' $snbVar(fn,seq)'"
		}

		usnb_log all "Making shifts '$snbVar(fn,shifts)' from '$snbVar(fn,seq)'"
		sys_run_makeshift $snbVar(fn,seq) $snbVar(fn,shifts)

		if [file readable $snbVar(fn,shifts)] {
			usnb_kill_win_nosave $f
			frame_snb_5 $f
		}
	}

	if { $snbVar(snbinit,constraints) == "orb" } {
		orb_main $snbVar(INSTALL) $snbVar(fn,session) 
		if { ![ catch { set a [glob "$snbVar(fn,session)/*.ppm"] } err]} {
			set snbVar(fn,shifts) [lindex $a 0]
			usnb_kill_win_nosave $f
			frame_snb_5 $f
		}
	}
}

######################################################################
#
# Init Screen 5 - get shift constraints 
#
proc frame_snb_5 { f } {
	global snbVar snbConst

	pack [frame $f] -fill x -expand true -anchor n -padx 20
    label $f.titleLabel -text "Enter Shift Constraints (5 of 6)"
    pack $f.titleLabel

	message $f.intro -text \
		"Enter an appropriate shifts constraints file\
		for smartnotebook. Please view the file before pressing\
		the CONTINUE button to make sure it is ok."
	pack $f.intro 

	pack [frame $f.file] -pady 15 
	u_file_entry $f.file.ent "Shift Constraints" \
		18 snbVar(fn,shifts) 36 $snbVar(snbwin,padx) { } 1 1

	# button widgets
	pack [frame $f.buttons] -pady 15
	pack [button $f.buttons.exe -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_snb_shifts $f"] -side left
	pack [button $f.buttons.back -text "Back" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_nosave $f; frame_snb_4 $f" ] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "abort_snb_init"] -side left
}

######################################################################
#
# Decide if shift constraints file is ok, move to the next screen
#
proc do_snb_shifts { f } {
	global snbVar snbConst

	if ![file readable $snbVar(fn,shifts)] {
		usnb_err "Cannot read shift constraints file ' $snbVar(fn,shifts)'"
	}

	usnb_kill_win_nosave $f
	frame_snb_6 $f
}

######################################################################
#
# Init Screen 6 - get optional files
#
proc frame_snb_6 { f } {
	global snbVar snbConst

	pack [frame $f] -fill x -expand true -anchor n -padx 20
    label $f.titleLabel -text "Optional Files (6 of 6)"
    pack $f.titleLabel

	message $f.intro -text \
		"The following optional settings allow creative use of smartnotebook.\
		Unless you exploring interesting ways of using this software\
		press the 'Finish Init' button to complete the initialization."
	pack $f.intro 
	pack [frame $f.fr] -pady 15 -fill both -expand true

	u_file_entry $f.fr.defaults "app defaults:" \
		16 snbVar(fn,defaults,user) 40 $snbVar(snbwin,padx) { } 1 1
    pack $f.fr.defaults 

	u_file_entry $f.fr.ppmout "ppm.out:" \
		16 snbVar(fn,ppm) 40 $snbVar(snbwin,padx) { } 1 1
    pack $f.fr.ppmout 

	# Have snb generate default connections 
	pack [frame $f.fr.connect] -anchor w
	label $f.fr.connect.label -text "Let snb calculate connectivities:" \
		-anchor e -width 36
	pack $f.fr.connect.label -side left -pady 5
	radiobutton $f.fr.connect.no -value 0 -text "no " \
		-variable snbVar(parm,calcConnect)
	pack $f.fr.connect.no -side left -padx 5
	radiobutton $f.fr.connect.yes -value 1 -text "yes " \
		-variable snbVar(parm,calcConnect)
	pack $f.fr.connect.yes -side left -padx 5

	# button widgets
	pack [frame $f.buttons] -pady 15 
	pack [button $f.buttons.save -text "Finish Init" \
		-padx $snbVar(snbwin,padx) \
		-command "do_snb_options $f"] -side left
	pack [button $f.buttons.back -text "Back" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_nosave $f; frame_snb_5 $f" ] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "abort_snb_init"] -side left
}

######################################################################
#
# Read option files and exit 
#
proc do_snb_options { f } {
	global snbVar snbConst

	if {$snbVar(parm,calcConnect) == 0 } {
		sys_remove_file "$snbVar(fn,session)/$snbVar(parm,c_out)*."
	}

	if ![file readable $snbVar(fn,defaults,user)] {
		usnb_err "User specified application defaults file does not exist"
	}
	if ![file exists $snbVar(fn,session)/$snbConst(F_DEFAULTS)] {
		sys_copy_file $snbVar(fn,defaults,user) $snbVar(fn,session)/$snbConst(F_DEFAULTS)
	}
	set snbVar(fn,defaults,user) $snbVar(fn,session)/$snbConst(F_DEFAULTS)
	usnb_kill_win_save $snbVar(snbinit,winName)
	set snbVar(OKFLAG) 0
}

######################################################################
######################################################################

######################################################################
#
# write out snb.init configuration file 
#
proc init_snb_write { fn } {
	global snbVar snbConst mySpectra

	if [catch {open $fn w} fp] {
		usnb_err "Cannot write to snb.init file: ' $fn'"
	}
	puts $fp "# "
	puts $fp "#  This is the user's customized init file as created"
	puts $fp "#  by the smartnotebook Initialization process. "
	puts $fp "# "

	puts $fp "  "
	puts $fp "# This is the user's selected experiment type, "
	puts $fp "# reference spectra, and spectral configuration type"
	puts $fp "set mySpectra(exp,type) $mySpectra(exp,type)"
	puts $fp "set mySpectra(exp,ref) $mySpectra(exp,ref)"
	puts $fp "set mySpectra(exp,spec) $mySpectra(exp,spec)"

	puts $fp "  "
	puts $fp "# Here is where we find the generic files for configuring the" 
	puts $fp "# above experiment selection."
	puts $fp "set mySpectra(fn,expInit,lib) $mySpectra(fn,expInit,lib)"
	puts $fp "set mySpectra(fn,nvInit,lib) $mySpectra(fn,nvInit,lib)"
	puts $fp "set mySpectra(fn,rulesdir) $mySpectra(fn,rulesdir)"
	puts $fp "set mySpectra(fn,searchItems) $mySpectra(fn,searchItems)"

	puts $fp "  "
	puts $fp "# Where do I find the input peaklists and processed spectra"
	for {set i 0} {$i < $mySpectra(xpk,fileNum)} {incr i} {
		if [file readable $mySpectra(xpk,defName,$i)] {
			puts $fp "set mySpectra(xpk,file,$i) $mySpectra(xpk,defName,$i)"
		}
	}
	for {set i 0} {$i < $mySpectra(nv,fileNum)} {incr i} {
		if [file readable $mySpectra(nv,defName,$i)] {
			puts $fp "set mySpectra(nv,file,$i) $mySpectra(nv,defName,$i)"
		}
	}

	puts $fp "  "
	puts $fp "# If there are imported atom assignments, where are they found"
	puts $fp "set snbVar(fn,ppm) $snbVar(fn,ppm)"

	puts $fp "  "
	puts $fp "# Output directory"
	puts $fp "set snbVar(fn,session) $snbVar(fn,session)" 

	puts $fp "  "
	puts $fp "# Web browser"
	puts $fp "set snbVar(fn,browser) $snbVar(fn,browser)" 

	puts $fp "  "
	puts $fp "# Applications defaults file"
	puts $fp "set snbVar(fn,defaults,user) $snbVar(fn,defaults,user)" 

	puts $fp "  "
	puts $fp "# Version control file"
	puts $fp "set snbVar(fn,version) $snbVar(fn,version)" 

	puts $fp "  "
	puts $fp "# The user can override the generic experiment defaults here."
	puts $fp "set mySpectra(fn,expInit,user) $mySpectra(fn,expInit,user)"
	puts $fp "set mySpectra(fn,nvInit,user) $mySpectra(fn,nvInit,user)"

	puts $fp "  "
	puts $fp "# Shift constraints file"
	puts $fp "set snbVar(fn,shifts) $snbVar(fn,shifts)"

	puts $fp "  "
	puts $fp "# Has user chosen to have snb generate connectivity files (0=no)?"
	puts $fp "set snbVar(parm,calcConnect) $snbVar(parm,calcConnect)"

	puts $fp "  "
	puts $fp "# Location of the user submitted connectivities file"
	puts $fp "set snbVar(fn,submit) $snbVar(fn,submit)"

	puts $fp "  "
	puts $fp "# Location of the final ppm.out file"
	puts $fp "set snbVar(fn,outppm) $snbVar(fn,outppm)"

	puts $fp "  "
	puts $fp "# Location of the chains file"
	puts $fp "set snbVar(fn,chains) $snbVar(fn,chains)"

	puts $fp "  "
	puts $fp "# Define a suggested chains file created by snb from assignments"
	puts $fp "# made in the reference peaklist."
	puts $fp "set snbVar(fn,suggest) $snbVar(fn,suggest)"
    close $fp
}

######################################################################
#
# user does not finish the init process 
#
proc abort_snb_init { } {
	global snbVar snbConst 
	sys_remove_file $snbVar(fn,snbInit,user)
	usnb_destroy_windows
}
