
######################################################################
#
# routines for the options menu item 
#
######################################################################

######################################################################
#
# Menu item which confirms that user voluntarily wants to go
# through init procedure
#
proc options_init { } {
	global snbVar snbConst varOption

	set f [usnb_show_win reinit "Configure / Update ..."]
	if { $f == "FALSE" } { return }

	set varOption(select) 1

	pack [frame $f.choice] -pady 10 -padx 20
	message $f.choice.intro -text \
		"Carefully select the appropriate update/initialization operation."
	pack $f.choice.intro

	pack [frame $f.choice.menu] -pady 10
	radiobutton $f.choice.menu.1 -value 1 -variable varOption(select) \
		-text "I have modified one or more peaklist files   "
	radiobutton $f.choice.menu.2 -value 2 -variable varOption(select) \
		-text "I have modified one or more connection files   "
	radiobutton $f.choice.menu.3 -value 3 -variable varOption(select) \
		-text "Please update nmrview peaklists with my snb assignments   "
	radiobutton $f.choice.menu.4 -value 4 -variable varOption(select) \
		-text "Initialize snb   "
	pack $f.choice.menu.1 $f.choice.menu.2 $f.choice.menu.3 \
		$f.choice.menu.4 -anchor w

	message $f.msg -text "" -width 200
	pack $f.msg -pady 5

	# buttons for this window
	frame $f.buttons
	pack $f.buttons -pady 20
	button $f.buttons.exe -text "Execute" \
		-padx $snbVar(snbwin,padx) \
		-command "do_options_init $f"
	button $f.buttons.quit -text "Cancel" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f"
	pack $f.buttons.exe $f.buttons.quit -side left 
}

######################################################################
#
# remove old initialization files, now when we restart snb we are
# prompted to enter info to set up new init files.
#
proc do_options_init { f } {
	global snbVar snbConst mySpectra varOption

	# save window state
	panel_state_disable
	usnb_save_geometry

	if { $varOption(select) == 1 } {
		$f.msg configure -text "Working..."
		snb_read_data $snbConst(E_XPK)
		$f.msg configure -text "Done"
	}

	if { $varOption(select) == 2 } {
		$f.msg configure -text "Working..."
		snb_read_data $snbConst(E_PEAKCON)
		$f.msg configure -text "Done"
	}

	if { $varOption(select) == 3 } {
		$f.msg configure -text "Working..."
		options_readpks_panel
		$f.msg configure -text "Done"
	}

	# re-init snb
	if { $varOption(select) == 4 } {
		usnb_log file "\n*** Backup Init File: '$snbVar(fn,snbInit,user)'"
		usnb_backup_file $snbVar(fn,snbInit,user) $snbVar(fn,log)
		usnb_log file "*** End Backup Init File: '$snbVar(fn,snbInit,user)'\n"
		sys_remove_file $snbVar(fn,snbInit,user)
		usnb_destroy_windows
		snb_startup
	}
	panel_state_enable
}

######################################################################
#
# user clicks on "Update nmrview" from the options menu
#
proc options_readpks_panel { } {
	global snbVar snbConst
	nvdraw_update_peaklists 1
}

######################################################################
# 
# User decides which spectra will have peak labels.
#
proc options_peaklabel {} {
	global snbVar snbConst mySpectra

	# if window exists, destroy it and start again.
	set f [usnb_show_win peaklabel "Spectra: Peak Labels"]
	if { $f == "FALSE" } { return }

	message $f.intro -text \
	"This window allows you to decide if labels for the individual \
	spectra should be drawn. Hiliting a box (by clicking) means 'yes'."
	pack $f.intro

	pack [frame $f.spectra]
	foreach w $mySpectra(winList) {
		pack [frame $f.spectra.$w] -pady 5 -anchor w
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			if [info exists mySpectra($w,$i,name)] {
		    	checkbutton $f.spectra.$w.$i -variable mySpectra($w,$i,label) \
					-text "$mySpectra($w,$i,name)   "
				pack $f.spectra.$w.$i -anchor w 
			}
	    }
	}

	# create buttons
	pack [frame $f.buttons] -side bottom -pady 20
	pack [button $f.buttons.save -text "Save \& Redraw" \
		-padx $snbVar(snbwin,padx) \
		-command "update_peaklabel"] -side left
	pack [button $f.buttons.cancel -text "Exit" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f; return"] -side left
}

######################################################################
#
# user wants to process change in peak label settings 
#
proc update_peaklabel { } {
	global snbVar snbConst

	# save parameters and re-draw screen 
	prefs_save_parms $snbConst(P_PEAKLABEL)
	usnb_update_spectra 0
}

######################################################################
#
# window for setting vertical scale
#
proc options_vscale {} {
	global snbVar snbConst mySpectra

	# if window exists, destroy it and start again.
	set f [usnb_show_win vscale "Spectra: Changing vertical scale"]
	if { $f == "FALSE" } { return }

	message $f.intro -text \
	"This window allows you to change the vertical scale for the individual \
	spectra. Move the slider and press 'Ok' to affect the change.\n\nTo change all sliders at once, hold down the Shift key while moving a slider.\n"
	pack $f.intro

	if ![info exists snbVar(vscale,from)] { set snbVar(vscale,from) 0.01 }
	if ![info exists snbVar(vscale,to)] { set snbVar(vscale,to) 1.00 }
	if ![info exists snbVar(vscale,res)] { set snbVar(vscale,res) 0.01 }
	if ![info exists snbVar(vscale,len)] { set snbVar(vscale,len) 2.00 }

	# create slider items
	pack [frame $f.slidemax] -pady 4
	u_command_entry $f.slidemax "Max slider value:" 17 \
		 "new_slider_max $f" -textvariable snbVar(vscale,to) -width 8

	pack [frame $f.spectra]
	foreach w $mySpectra(winList) {
		pack [frame $f.spectra.$w] -pady 1 -anchor w
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			if [info exists mySpectra($w,$i,name)] {
				u_slider_entry $f.spectra.$w.$i mySpectra($w,$i,lvl) \
					"$mySpectra($w,$i,name):" $snbVar(vscale,from) \
					$snbVar(vscale,to) $snbVar(vscale,res) $snbVar(vscale,len)
				set cmd "bind $f.spectra.$w.$i.scale <Shift-ButtonPress-1> \
					{ change_all_sliders %W mySpectra($w,$i,lvl) }"
				eval $cmd
			}
	    }
	}

	# create buttons
	pack [frame $f.buttons] -side bottom -pady 8
	pack [button $f.buttons.save -text "Ok" \
		-padx $snbVar(snbwin,padx) \
		-command "update_vscale"] -side left
	pack [button $f.buttons.cancel -text "Exit" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $f; return"] -side left
}

######################################################################
#
# when vertical scale sliders are changed, this is called
# to set all sliders to the same value
#
proc change_all_sliders { field var } {

	# now set a binding until the KeyRelease happens
	set cmd "bind $field <Motion> {change_all_sliders2 $var }"
	eval $cmd

	# undo previous binding
	set cmd "bind $field <ButtonRelease-1> \
		{ change_all_sliders2 $var; 
		bind $field <Motion> \"\" ;
		bind $field <ButtonRelease-1> \"\"
		}" 
	eval $cmd 
}

proc change_all_sliders2 { var } {
global mySpectra

	set f ".snbmain.vscale"
	upvar $var varname
	set lvl $varname

	foreach w $mySpectra(winList) {
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			$f.spectra.$w.$i.scale set $lvl
		}
	}
}


######################################################################
#
# user wants to process change in vertical scale values 
#
proc new_slider_max { f } {
	global snbVar snbConst mySpectra

	foreach w $mySpectra(winList) {
		for {set i 0} {$i < $mySpectra($w,spectraNum)} {incr i} {
			if [info exists mySpectra($w,$i,name)] {
				$f.spectra.$w.$i.scale configure -to $snbVar(vscale,to)
			}
		}
	}
}

######################################################################
#
# user wants to process change in vertical scale values 
#
proc update_vscale { } {
	global snbVar snbConst mySpectra

	# save parameters and re-draw screen 
	prefs_save_parms $snbConst(P_VSCALE)
	source $mySpectra(fn,nvInit,user)
	usnb_update_spectra 0
}

######################################################################
#
# preferences window for colors 
#
proc options_settings {} {
	global snbVar snbConst

	# if window exists, destroy it and start again.
	set f [usnb_show_win preferences "Preferences"]
	if { $f == "FALSE" } { return }

	frame $f.snbfont
	pack $f.snbfont -pady 10
	label $f.snbfont.boldLabel -text "Window Font"
	pack $f.snbfont.boldLabel
	u_list_entry $snbVar(fonts) $f.snbfont.family snbVar(window,fontFamily) "Font Family:"
	u_list_entry $snbVar(fontSizes) $f.snbfont.size snbVar(window,fontSize) "Font Size:"

	frame $f.bfitfont
	pack $f.bfitfont -pady 10 -padx 10
	label $f.bfitfont.boldLabel -text "Fixed font for text displayed in columns"
	pack $f.bfitfont.boldLabel
	u_list_entry $snbVar(fixedFonts) $f.bfitfont.family snbVar(text,fontFamily) "Font Family:"
	u_list_entry $snbVar(fontSizes) $f.bfitfont.size snbVar(text,fontSize) "Font Size:"

	pack [frame $f.misc] -pady 20
	label $f.misc.boldLabel -text "Other Attributes "
	pack $f.misc.boldLabel
	u_text_entry $f.misc.1 snbVar(snbmain,colMax) "Amino Acid Columns" 3

	# create buttons
	pack [frame $f.buttons] -side bottom -pady 20
	pack [button $f.buttons.save -text "Ok" \
		-padx $snbVar(snbwin,padx) \
		-command "settings_ok"] -side left
	pack [button $f.buttons.reset -text "Defaults" \
		-padx $snbVar(snbwin,padx) \
		-command "settings_reset"] -side left
	pack [button $f.buttons.cancel -text "Exit" \
		-padx $snbVar(snbwin,padx) \
		-command  "usnb_kill_win_save $f; return"] -side left
}

######################################################################
#
#  show new setings 
#
proc settings_ok {} {
	global snbVar snbConst

	prefs_save_parms $snbConst(P_SETTINGS)
	panel_restart_snb
	options_settings
}

######################################################################
#
#  reset colors for sequence panel 
#
proc settings_reset {} {
	global snbVar snbConst

	source $snbVar(INSTALL)/lib/$snbConst(F_DEFAULTS)
	prefs_save_parms $snbConst(P_SETTINGS)
	panel_restart_snb
	options_settings
}
