
######################################################################
#
# search for connections subroutines
#
######################################################################

######################################################################
#
# search panel section 
#
proc search_terms { } {
	global snbVar snbConst varSearch mySpectra

	set win [usnb_show_win search "Search Connectivities"]
	if { $win == "FALSE" } { return }
	init_search_vars

	set g $win.buttons
	pack [frame $g] -pady 10 -padx 10
	pack [label $g.boldLabel -text "Residue ID Constraints"]
	pack [frame $g.top]
	pack [frame $g.bot]

	# ResId constraints
	radiobutton $g.top.1 -value $snbConst(S_NONE) \
		-font $snbVar(window,font) -width 9 \
		-text "none   " -variable varSearch(cResId)
	radiobutton $g.bot.1 -value $snbConst(S_CURRENT) \
		-font $snbVar(window,font) -width 9 \
		-text "current" -variable varSearch(cResId)
	radiobutton $g.top.2 -value $snbConst(S_NEXT_UNUSED) \
		-font $snbVar(window,font) -width 14 \
		-text "Next Un-used" -variable varSearch(cResId)
	radiobutton $g.bot.2 -value $snbConst(S_PREV_UNUSED) \
		-font $snbVar(window,font)  -width 14 \
		-text "Prev Un-used" -variable varSearch(cResId)
	radiobutton $g.top.3 -value $snbConst(S_NEXT_UNASSIGN) \
		-font $snbVar(window,font)  -width 18 \
		-text "Next Un-assigned" -variable varSearch(cResId)
	radiobutton $g.bot.3 -value $snbConst(S_PREV_UNASSIGN) \
		-font $snbVar(window,font)  -width 18 \
		-text "Prev Un-assigned" -variable varSearch(cResId)
	pack $g.top.1 $g.top.2 $g.top.3 -side left
	pack $g.bot.1 $g.bot.2 $g.bot.3 -side left

	# Atom constraints section
	set f $win.resid
	pack [frame $f] -pady 10
	pack [label $f.boldLabel -text "Atom Constraints"]
	pack [frame $f.fr1] -pady 0 
	pack [frame $f.fr2] -pady 10 
	pack [frame $f.fr3] -pady 0
	frame $f.fr1.0 -relief raised -bd 2
	frame $f.fr1.1 -relief raised -bd 2
	pack $f.fr1.0 $f.fr1.1 -pady 3

	foreach pSide "0 1" {
		set box $f.fr1.$pSide
		set varSearch(gui,$pSide) $box

		pack [frame $box.title]
		label $box.title.resid -font $snbVar(window,font) -width 12 \
			-text $snbConst($pSide,S_TITLE)
		label $box.title.label -textvariable varSearch($pSide,resId) \
			-font $snbVar(window,font)
		pack $box.title.resid $box.title.label  -padx 10  -side left

		frame $box.aa -bd 1
		set varSearch(list,$pSide) [listbox $box.aa.listing \
			-setgrid false -selectmode browse -width 8 -height 5 \
			-font $snbVar(window,font) \
		    -yscrollcommand "$box.aa.scroll set"]
	    scrollbar $box.aa.scroll -command "$box.aa.listing yview"
		pack $box.aa.scroll -side right -fill y 
	    pack $box.aa.listing -side left -fill both -expand true
	    bind $varSearch(list,$pSide) <ButtonRelease-1> \
			[list event_aa_constraints $pSide $varSearch(list,$pSide) %y]

		frame $box.atoms
		foreach atom $mySpectra(atoms,names) {
			pack [frame $box.atoms.$atom] -padx 10
			pack [entry $box.atoms.$atom.lb -textvariable \
				varSearch($pSide,$atom,lb) -width 6 -relief sunken -font \
				$snbVar(window,font) ] -anchor w -side left
			pack [label $box.atoms.$atom.scor -text ">= $atom >=" \
				-width 9 -font $snbVar(window,font)] -side left
			pack [entry $box.atoms.$atom.ub -textvariable \
				varSearch($pSide,$atom,ub) -width 6 -font \
				$snbVar(window,font) -relief sunken] -anchor w -side left
		}
		pack $box.aa $box.atoms -side left -padx 5 -pady 3
	}

	# probability score constraints
	label $f.fr2.boldLabel -text "Probability Score:"
	pack $f.fr2.boldLabel -anchor w -side left
	frame $f.fr2.box
	pack $f.fr2.box -anchor w -padx 20 -pady 0 -side left
	pack [entry $f.fr2.box.lb -textvariable varSearch(cProb,lb) \
		-font $snbVar(window,font) \
		-width 5 -relief sunken] -anchor w -side left
	pack [label $f.fr2.box.scor -text ">= Score >=" \
		-font $snbVar(window,font)] -side left
	pack [entry $f.fr2.box.ub -textvariable varSearch(cProb,ub) \
		-font $snbVar(window,font) \
		-width 5 -relief sunken] -anchor w -side left

	# Number of hits message
	label $f.fr3.hits -text $varSearch(hitLabel) \
		-font $snbVar(window,font) 
	pack $f.fr3.hits
	set varSearch(gui,hits) $f.fr3.hits

	# buttons at the bottom
	set g $win.exit
	pack [frame $g] -pady 5
	pack [button $g.searchButton -text "Search" \
		-font $snbVar(window,font) \
		-padx $snbVar(snbwin,padx) \
		-command "event_search_button"] -side left
	pack [button $g.quitButton \
		-font $snbVar(window,font) \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $win"] -side left

	# set starting values for atom constraints

	# get latest settings for default atom search constraints	
	read_search_terms $mySpectra(fn,searchItems)
	foreach pSide "0 1" {
		foreach aa $snbConst(AMINO_ACIDS) {
			$varSearch(list,$pSide) insert end $aa
			set_aa_constraints $pSide 0
		}
	}
}

######################################################################
#
#  init variables of the search window unless already set
#
proc init_search_vars { } {
	global snbVar snbConst varSearch mySpectra

	u_set_val varSearch(cResId) $snbConst(S_NONE)

	foreach pSide "0 1" {
		u_set_val varSearch($pSide,resId)  " "
		u_set_val varSearch($pSide,resCurrent)  "-"
		u_set_val varSearch($pSide,curResId) 0
	}

	u_set_val varSearch(cProb,lb) 0.0
	u_set_val varSearch(cProb,ub) 1.0
	u_set_val varSearch(hitLabel) " "
}

######################################################################
#
# user selects "search" button 
#
proc event_search_button { } {
	global snbVar snbConst varSearch 

	set varSearch(hits) 0
	$varSearch(gui,hits) configure -text " "
	update

	if { $varSearch(cResId) == $snbConst(S_NEXT_UNUSED) } {
		next_unused 1
	}
	if { $varSearch(cResId) == $snbConst(S_PREV_UNUSED) } {
		next_unused -1
	}
	if { $varSearch(cResId) == $snbConst(S_NEXT_UNASSIGN) } {
		next_unassign 1
	}
	if { $varSearch(cResId) == $snbConst(S_PREV_UNASSIGN) } {
		next_unassign -1
	}
	if { $varSearch(cResId) == $snbConst(S_CURRENT) } {
		parse_connect $snbVar(panel,refId)
	}
	if { $varSearch(cResId) == $snbConst(S_NONE) } {
		parse_connect -1 
	}

	# update "search hits"
	set varSearch(hitLabel) [format "%s: %3d" "Search Hits" $varSearch(hits)]
	$varSearch(gui,hits) configure -text $varSearch(hitLabel) 

	# update connection panel on main snb window
	connect_panel_update
}

######################################################################
#
# user selects "next un-used" or "prev un-used" Reference Id constraint 
#
proc next_unused { inc } {
	global snbVar snbConst mySpectra myPeak myConnect myChain

	set xpkix $mySpectra(REF,xpkIx)
	get_starting_resid

	# look for the next available refId for search/display 
	set ref0 $snbVar(panel,lastRef)
	set count 0
	while { $count < $myConnect(refIdMax) } {
		set ref0 [expr $ref0 + $inc]
		if { $inc > 0 && $ref0 > $myConnect(refIdMax) } {
			set ref0 0
		}
		if { $inc < 0 && $ref0 < 0 } {
			set ref0 $myConnect(refIdMax) 
		}
		if { [info exists myChain($ref0,chainId)] } {
			if { [llength $myChain($ref0,chainId)] <= 1 } {
				break
			}
		}
		incr count
	}

	# search for connections
	parse_connect $ref0
	set snbVar(panel,refId) $ref0

	# update reference spectra
	set iPeak $myPeak($xpkix,index,$ref0)
	set peakVal $myPeak($xpkix,origPeak,$iPeak)
	set x2 [lindex $peakVal $mySpectra(REF,xdim)]
	set y2 [lindex $peakVal $mySpectra(REF,ydim)]
	nvdraw_ref [list $snbConst(NO_VAL) $snbConst(NO_VAL) $x2 $y2 ]
}

######################################################################
#
# user selects "next un-assign" or "prev un-assign" Reference Id constraint 
#
proc next_unassign { inc } {
	global snbVar snbConst mySpectra myPeak myConnect

	set xpkix $mySpectra(REF,xpkIx)
	get_starting_resid

	# look for the next available refId for search/display 
	set ref0 $snbVar(panel,lastRef)
	set count 0
	while { $count < $myConnect(refIdMax) } {
		set ref0 [expr $ref0 + $inc]
		if { $inc > 0 && $ref0 > $myConnect(refIdMax) } {
			set ref0 0
		}
		if { $inc < 0 && $ref0 < 0 } {
			set ref0 $myConnect(refIdMax) 
		}
		set iPeak $myPeak($xpkix,index,$ref0)
		if ![info exists myPeak($xpkix,assignLine,$iPeak)] {
			break
		}
		incr count
	}

	# search for connections
	parse_connect $ref0
	set snbVar(panel,refId) $ref0

	# update reference spectra
	set peakVal $myPeak($xpkix,origPeak,$iPeak)
	set x2 [lindex $peakVal $mySpectra(REF,xdim)]
	set y2 [lindex $peakVal $mySpectra(REF,ydim)]
	nvdraw_ref [list $snbConst(NO_VAL) $snbConst(NO_VAL) $x2 $y2 ]
} 

######################################################################
#
# Search connection records based on user determined search criteria 
#
proc parse_connect { refid } {
	global snbVar snbConst myConnect mySearch mySpectra varSearch

	# delete previous connections search list 
	set mySearch(foundList) [lreplace mySearch(foundList) 0 end] 

	# test each connectivity record 
	set iProb $mySpectra(connect,PROB)
	set cNum $myConnect(connectNum)
	for {set cix 1} {$cix <= $cNum} {incr cix} {

		# resId constraints
		if { $refid >= 0 } { 
			if {$myConnect($cix,ref,0) != $refid && \
				$myConnect($cix,ref,1) != $refid} { continue }
		}

		# test each variation of refId0-refId1 connectivities
		set pNum $myConnect($cix,pNum)
		for {set pix 1} {$pix <= $pNum} {incr pix} {

			# Probability constraints
			set prob $myConnect($cix,$pix,$iProb)
			if { $prob < $varSearch(cProb,lb) } { break }
			if { $prob > $varSearch(cProb,ub) } { break }

			# atom constraints
			set flag 0
			foreach pSide "0 1" {
				foreach atomName $mySpectra(atoms,names) {
					set lb $varSearch($pSide,$atomName,lb)
					set ub $varSearch($pSide,$atomName,ub)
					set atom $mySpectra(connect,$atomName,$pSide)
					set shift $myConnect($cix,$pix,$atom)
					if {$shift != $snbConst(NO_VAL) } {
						if {$shift < $lb || $shift > $ub} { 
							set flag 1
							break
						}
					} else {
						if {$lb != $snbConst(NO_VAL)} { 
							set flag 1
							break
						}
					}
				}
				if { $flag == 1 } { break }
			}
			if { $flag == 1 } { break }

			# all constraints satisfied, add to our list
			lappend mySearch(foundList) "$prob 1 $cix $pix"
			incr varSearch(hits)
		}
	}

	# return sorted list of hits based on prob score
	set mySearch(foundList) [lsort -decreasing $mySearch(foundList)]
}

######################################################################
#
# make sure we have a starting point when searching for next
# un-assigned or un-used amino acid.
#
proc get_starting_resid { } {
	global snbVar snbConst 

	# make sure we have a starting point
	if { $snbVar(panel,refId) == "" } {
		if { ![info exist snbVar(panel,lastRef)] } {
			set snbVar(panel,lastRef) 0
		}
	} else {
		set val [string first - $snbVar(panel,refId)]
		if { $val >= 0 } {
			set snbVar(panel,lastRef) [string range $snbVar(panel,refId) 0 [expr $val - 1]]
		} else {
			set snbVar(panel,lastRef) $snbVar(panel,refId)
		}
	}
}

######################################################################
#
# Set shift constraints for ResID(i) or (i-1) based on selected amino
# acid type
#
proc event_aa_constraints { pSide list y } {
	global snbVar snbConst varSearch mySpectra mySeq

	# Remember which is the current resId box
	set varSearch($pSide,resId) " "
	set ix [$varSearch(list,$pSide) nearest $y]

	if { $ix <= $varSearch(num) } {
		set varSearch($pSide,curResId) 1
		set_aa_constraints $pSide $ix
		return
	}

	set aLabel [$varSearch(list,$pSide) get $ix] 
	set resName [string range $aLabel 0 0]

	if { $resName == $varSearch($pSide,resCurrent) } {
		incr varSearch($pSide,curResId)
	} else {
		set varSearch($pSide,curResId) 1
	}

	foreach id "$varSearch($pSide,curResId) 1" {
		for {set aaIx $id} {$aaIx <= $mySeq(seqLen)} {incr aaIx} {
			set name1 $mySeq($aaIx,name1)
			if { $name1 == $resName } {
				set varSearch($pSide,curResId) $aaIx 
				set varSearch($pSide,resCurrent) $resName
				search_atom_constrain $aaIx $pSide
				return	
			}
		}
	}
}


######################################################################
#
# Set shift constraints for ResID(i) or (i-1) based on selected amino
# acid type
#
proc set_aa_constraints { pSide ix } {
	global snbVar snbConst varSearch mySpectra

	foreach aName $mySpectra(atoms,names) {
		if [info exists varSearch(list,$ix,$aName,lb)] {
			if [info exists varSearch(list,$ix,$aName,ub)] {
				set varSearch($pSide,$aName,lb) $varSearch(list,$ix,$aName,lb)
				set varSearch($pSide,$aName,ub) $varSearch(list,$ix,$aName,ub)
				$varSearch(gui,$pSide).atoms.$aName.lb configure -state normal 
				$varSearch(gui,$pSide).atoms.$aName.scor configure -state normal
				$varSearch(gui,$pSide).atoms.$aName.ub configure -state normal 
				continue
			}
		}
		$varSearch(gui,$pSide).atoms.$aName.lb configure -state disabled 
		$varSearch(gui,$pSide).atoms.$aName.scor configure -state disabled 
		$varSearch(gui,$pSide).atoms.$aName.ub configure -state disabled 
		set varSearch($pSide,$aName,lb) $snbConst(NO_VAL) 
		set varSearch($pSide,$aName,ub) $snbConst(NO_VAL)
	}
}

######################################################################
######################################################################
######################################################################

######################################################################
#
# Set shift constraints for ResID(i) or (i-1) 
# This can be done through sequence panel or search window
#
proc search_atom_constrain { aaIx pSide } {
	global snbVar snbConst mySeq mySpectra varSearch

	# search window must exist for this to work 
	if ![winfo exists $snbVar(search,winName)] {
		return
	}

	# if called from search window, then make sure amino acid is in our
	# sequence.
	if ![info exists mySeq($aaIx,name3)] {
		return
	}

	set name3 $mySeq($aaIx,name3)
	set aaNum $mySeq($aaIx,aaNum)
	set varSearch($pSide,resId) [format "%3s:%3d" $name3 $aaNum]

	# process each atom of the residue
	foreach atomName $mySpectra(atoms,names) {
		if [info exists mySeq($aaIx,$atomName,mean)] {
			set mean $mySeq($aaIx,$atomName,mean)
			set sdev $mySeq($aaIx,$atomName,sdev)
			set varSearch($pSide,$atomName,lb) [expr $mean - $snbVar(bestfit,sdevTerm) * $sdev]
			set varSearch($pSide,$atomName,ub) [expr $mean + $snbVar(bestfit,sdevTerm) * $sdev]
			$varSearch(gui,$pSide).atoms.$atomName.lb configure -state normal 
			$varSearch(gui,$pSide).atoms.$atomName.scor configure -state normal 
			$varSearch(gui,$pSide).atoms.$atomName.ub configure -state normal 
		} else {
			set varSearch($pSide,$atomName,lb) $snbConst(NO_VAL) 
			set varSearch($pSide,$atomName,ub) $snbConst(NO_VAL) 
			$varSearch(gui,$pSide).atoms.$atomName.lb configure -state disabled 
			$varSearch(gui,$pSide).atoms.$atomName.scor configure -state disabled 
			$varSearch(gui,$pSide).atoms.$atomName.ub configure -state disabled 
		}
	}
}

######################################################################
#
# user selects "spectra" button 
# find closest reference peak to the crosshairs in hsqc spectra 
#
proc search_from_spectra { } {
	global snbVar snbConst mySpectra myPeak

	# find item in our peaklist
	set xpkix $mySpectra(REF,xpkIx)
	set iPeak [ calc_closest_peak $xpkix ]
	if { $iPeak < 0 } {
		usnb_err "Please click on a peak in the hsqc spectra first. "
	}
	set snbVar(panel,refId) $myPeak($xpkix,pkId,$iPeak)

	# search connections
	search_ref_ids $snbVar(panel,refId) $snbVar(panel,refId) $snbConst(P_EMPTY) 1

	# update reference spectra
	set peakVal $myPeak($xpkix,origPeak,$iPeak)
	set x2 [lindex $peakVal $mySpectra(REF,xdim)]
	set y2 [lindex $peakVal $mySpectra(REF,ydim)]
	nvdraw_ref [list $snbConst(NO_VAL) $snbConst(NO_VAL) $x2 $y2 ]
} 

######################################################################
#
# user enters a reference peak id or ids 
#
proc search_from_keyboard { } {
	global snbVar snbConst

	# an empty field means there is nothing to do. 
	if { $snbVar(panel,refId) == "" } { return }

	set connector [ string first - $snbVar(panel,refId) ]
	if { $connector >= 0} {
		# user has entered multiple refIds
		set ref1 [string range $snbVar(panel,refId) 0 [expr $connector - 1]]
		set ref2 [string range $snbVar(panel,refId) [expr $connector + 1] end]
		search_ref_ids $ref1 $ref2 $snbConst(P_EMPTY) 2

	} else {
		# search for connections with one refId
		search_ref_ids $snbVar(panel,refId) $snbVar(panel,refId) $snbConst(P_EMPTY) 1
	}

	# update reference spectra
	set xy [usnb_find_ref_values 0 $snbVar(panel,refId)]
	nvdraw_ref $xy
} 

######################################################################
#
# search connection files for reference ids 
#
proc search_ref_ids { ref1 ref2 fitProc matchNum } {
	global snbVar snbConst mySearch

	panel_state_disable	

# DEBUG code:  timed search 
# set val [ time "do_connect_search $ref1 $ref2 $fitProc $matchNum" 1 ]
# puts stdout "time = $val"
	do_connect_search $ref1 $ref2 $fitProc $matchNum

	# update connection panel
	connect_panel_update

	usnb_log file "connect search: $ref1 $ref2 $fitProc $matchNum: $mySearch(searchNum)"
	panel_state_enable	
}

######################################################################
#
# Search connection files for reference Ids 
#
proc do_connect_search { pidFrom pidTo fitProc matchNum } {
	global snbVar snbConst myChain myConnect mySearch mySeq
	global csParms mySpectra

	# perhaps we just repeat the previous search with a different pattern
	# if no previous search, do nothing
	if { $fitProc == $snbConst(P_PREV) } {
		if [ info exists csParms(pidFrom) ] {
			set pidFrom $csParms(pidFrom) 
			set pidTo $csParms(pidTo)
			set fitProc $csParms(fitProc)
			set matchNum $csParms(matchNum)
		} else {
			return 0
		}
	}

	# keep track of our current search parameters
	set csParms(pidFrom) $pidFrom
	set csParms(pidTo) $pidTo
	set csParms(fitProc) $fitProc
	set csParms(matchNum) $matchNum

	# delete previous connections search list 
	set mySearch(foundList) [lreplace mySearch(foundList) 0 end] 

	# find the new matches (ie, do the connectivity search)
	set iProb $mySpectra(connect,PROB)
	set cNum $myConnect(connectNum)
	for {set cix 1} {$cix <= $cNum} {incr cix} {
		set rule $myConnect($cix,rule)
		if { $snbVar(panel,ruleSelect) == "all" || $snbVar(panel,ruleSelect) == $rule } {
			set count 0
			if { $myConnect($cix,ref,0) == $pidFrom } { incr count }
			if { $myConnect($cix,ref,1) == $pidTo } { incr count }
			if { $count < $matchNum } { continue }

			# we have a match
			if { $fitProc == $snbConst(P_START) } { fit_to_start_chain $cix }
			if { $fitProc == $snbConst(P_MID) } { fit_to_middle_chain $cix }
			if { $fitProc == $snbConst(P_END) } { fit_to_end_chain $cix }
			if { $fitProc == $snbConst(P_EMPTY) } { fit_to_empty_chain $cix }
		}
	}

	# return sorted list
	set mySearch(foundList) [lsort -decreasing $mySearch(foundList)]
}

######################################################################
#
# find closest peak to the crosshairs in active spectra
# (reference spectra only for now)
#
proc calc_closest_peak { xpkix } {
	global snbVar snbConst myPeak mySpectra
	
	set bestdiff 99999
	set bestIx -1
	set num $myPeak($xpkix,nPeak)

	set x0 [ nv_win cross1x ]
	set y0 [ nv_win cross1y ]
	set xScale [ lindex $myPeak($xpkix,sw) $mySpectra(REF,xdim) ]
	set yScale [ lindex $myPeak($xpkix,sw) $mySpectra(REF,ydim) ]

	# try to compensate for different sweep width on each axis
	for {set iPeak 1} {$iPeak <= $num} {incr iPeak} {
		set thisPeak $myPeak($xpkix,origPeak,$iPeak)
		set x1 [ lindex $thisPeak $mySpectra(REF,xdim) ]
		set y1 [ lindex $thisPeak $mySpectra(REF,ydim) ]
		set x2 [ expr (($x1 - $x0) * $yScale) * (($x1 - $x0) * $yScale) ]
		set y2 [ expr (($y1 - $y0) * $xScale) * (($y1 - $y0) * $xScale) ]
		set diff [ expr $x2 + $y2 ]
		if { $diff < $bestdiff } {
			set bestIx $iPeak
			set bestdiff $diff
		}
	}
	return $bestIx
}

######################################################################
#
# read default search queries for the selected experiment type 
#
proc read_search_terms { fname } {
	global snbVar snbConst varSearch

	set num -1 
	set varSearch(num) $num

	if {[ catch { set fn [ open $fname r ] } err]} {
		return
	}

	if [info exists varSearch(list,0)] {
		$varSearch(list,0) delete 0 end
		$varSearch(list,1) delete 0 end
	}

	while {[gets $fn line] >= 0} {
		if { [string index $line 0] == $snbConst(COMMENT)} {
			continue
		}

		set nd [scan $line "%s %s %s %s" key aLabel lb ub]
		if {$nd >= 2 } {
			if { [string compare $key "Label:"] == 0 } {
				incr num
				$varSearch(list,0) insert end $aLabel
				$varSearch(list,1) insert end $aLabel
			}

			if { [string compare $key "Atom:"] == 0 } {
				set varSearch(list,$num,$aLabel,lb) $lb
				set varSearch(list,$num,$aLabel,ub) $ub
			}
		}
	}
	set varSearch(num) $num
	close $fn
}
