
######################################################################
#
#  user submitted connection creation routines 
#
######################################################################


######################################################################
#
# display main connection creation window 
#
proc submit_add_main { } {
	global snbVar snbConst varSubmit

	set f [usnb_show_win submit1 "Creating a new connection record"]
	if { $f == "FALSE" } { return }

	# start with first frame
	set varSubmit(win) $f
	set varSubmit(winNum) 1
	frame_submit_refs $f.frame
}

######################################################################
#
# Screen 1:  display main connection creation window 
#
proc frame_submit_refs { f } {
	global snbVar snbConst varSubmit

	pack [frame $f] -fill x -expand true -anchor n -padx 20
	label $f.title -text "Setting reference Ids (step 1 of 4)"
	pack $f.title

	pack [frame $f.1] -fill x -expand true
	message $f.1.intro -text \
		"Enter the reference spectra Ids of the proposed connection. Press\
		CONTINUE to go to the next step." 
	pack $f.1.intro

	pack [frame $f.ref0] -pady 10
	pack [frame $f.ref1] -pady 10
	u_command_entry $f.ref0 "RefId(i-1):" 12 \
			"" -textvariable varSubmit(cRec,0) -width 4 
	u_command_entry $f.ref1 "RefId(i):" 12 \
			"" -textvariable varSubmit(cRec,1) -width 4 

	pack [frame $f.buttons] -pady 15 -padx 10
	pack [button $f.buttons.cont -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_submit_next_window $f 1"] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $varSubmit(win)"] -side left
}

######################################################################
#
# Screen 2: Enter chemical shifts in connection 
#
proc frame_submit_atoms { f } {
	global snbVar snbConst varSubmit mySpectra

	# reference Id
	set ix $varSubmit(resIx)
	set refId $varSubmit(cRec,$ix)

	pack [frame $f] -fill x -expand true -anchor n -padx 20
	label $f.title -text "Set shift values for reference ID $refId\n\
		(step $varSubmit(winNum) of 4)"
	pack $f.title

	pack [frame $f.1] -fill x -expand true
	message $f.1.intro -text \
		"First, if you know the correct peakIds from the\
		$mySpectra(xpk,desc,1) peaklist\
		enter these values via the keyboard or click the item in\
		the list box.\
		Note that\
		'$snbConst(NO_VAL)' is designated as the NULL value."
	pack $f.1.intro

	pack [frame $f.pids]
	frame $f.pids.curSel
	frame $f.pids.listWin
	pack $f.pids.curSel $f.pids.listWin -side left -pady 20 -padx 20

	# current peakid selection section
	label $f.pids.curSel.label -text "Current PeakIds"
	pack $f.pids.curSel.label
	foreach peakName $mySpectra(connect,ATOMS,TITLE) {
		pack [frame $f.pids.curSel.x$peakName] -pady 0
		u_command_entry $f.pids.curSel.x$peakName "$peakName" 5 \
			"submit_ids_kbd" -textvariable varSubmit($ix,peak,$peakName) \
			-width 8 
	}

	# listbox of peakid selections
	label $f.pids.listWin.label -text $mySpectra(connect,ATOMS,TITLE)
	pack $f.pids.listWin.label -anchor w
	set varSubmit($ix,pkIdList) [listbox $f.pids.listWin.listing \
		-selectmode single\
		-yscrollcommand "$f.pids.listWin.scroll set"]
	scrollbar $f.pids.listWin.scroll -command "$f.pids.listWin.listing yview"
	pack $f.pids.listWin.scroll -side right -fill y
	pack $f.pids.listWin.listing -side left -fill both -expand true
	bind $varSubmit($ix,pkIdList) <ButtonRelease-1> \
		[list submit_ids_menu $varSubmit($ix,pkIdList) %y]

	pack [frame $f.atoms] -pady 10
	message $f.atoms.intro -text \
		"Next, specify the atom shift values associated with reference\
		ID $refId. Correct any displayed values."
	pack $f.atoms.intro
	foreach atomName $mySpectra(atoms,names) {
		pack [frame $f.atoms.x$atomName] -pady 0
		u_command_entry $f.atoms.x$atomName "$atomName:" 5\
			"" -textvariable varSubmit($ix,atom,$atomName) -width 8 
	}

	pack [frame $f.buttons] -pady 15 -padx 10 -side bottom
	pack [button $f.buttons.cont -text "Continue" \
		-padx $snbVar(snbwin,padx) \
		-command "do_submit_next_window $f 1"] -side left
	pack [button $f.buttons.back -text "Back" \
		-padx $snbVar(snbwin,padx) \
		-command "do_submit_next_window $f -1"] -side left
	pack [button $f.buttons.clear -text "Clear" \
		-padx $snbVar(snbwin,padx) \
		-command "do_clear_atoms"] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $varSubmit(win)"] -side left

	# show current choices for user
	set num [scan_connect $refId $ix ]
}

######################################################################
#
# process CONTINUE or BACK buttons in first 3 screens
#
proc do_submit_next_window { f dir } {
	global snbVar snbConst varSubmit mySpectra

	usnb_kill_win_save $f
	if {$dir > 0 } {

		# turn on labels, show spectra if this was first screen
		if {$varSubmit(winNum) == 1} {
			set snbVar(panel,refId) "$varSubmit(cRec,0)-$varSubmit(cRec,1)"
			for {set i 0} {$i < $mySpectra(strips,spectraNum)} {incr i} {
				set mySpectra(strips,$i,prevLabel) $mySpectra(strips,$i,label) 
				set mySpectra(strips,$i,label) 1
			}
			set xy [ usnb_find_ref_values 1 $snbVar(panel,refId) ]
			nvdraw_ref $xy
			for {set i 0} {$i < $mySpectra(strips,spectraNum)} {incr i} {
				set mySpectra(strips,$i,label) $mySpectra(strips,$i,prevLabel) 
			}
		}

		set varSubmit(resIx) [expr $varSubmit(winNum) - 1] 
		incr varSubmit(winNum)

		if {$varSubmit(winNum) > 3} {
			frame_submit_save $f
		} else {
			frame_submit_atoms $f
		}
	} else {
		set varSubmit(winNum) [expr $varSubmit(winNum) - 1]
		set varSubmit(resIx) [expr $varSubmit(resIx) - 1] 
		if { $varSubmit(winNum) == 1 } {
			frame_submit_refs $f
		} else {
			frame_submit_atoms $f
		}
	}
}

######################################################################
#
# Search connection records for a given reference Id.
# Return peakids and corresponding atom values 
#
proc scan_connect { refId iPos } {
	global snbVar snbConst varSubmit myConnect myPeak mySpectra

	$varSubmit($iPos,pkIdList) delete 0 end
	set varSubmit($iPos,totList) {} 
	set matchNum 0
	set cNum $myConnect(connectNum)
	for {set cix 1} {$cix <= $cNum} {incr cix} {

		if { $myConnect($cix,ref,$iPos) != $refId } {
			continue
		}

		# get peakids which have relevant shift values
		set pkList "" 
		foreach field $mySpectra(connect,PKID,$iPos) {
			set pk [format "%-5s" $myConnect($cix,1,$field)]
			set pkList "$pkList$pk"
		}

		# have we seen this combination of peakids before
		if { [lsearch $varSubmit($iPos,totList) $pkList] >= 0 } {
			continue
		}

		lappend varSubmit($iPos,totList) $pkList
		$varSubmit($iPos,pkIdList) insert end $pkList

		# start with no shift values
		foreach atom $mySpectra(atoms,names) {
			set varSubmit($iPos,$matchNum,$atom) $snbConst(NO_VAL) 
		}

		# find xpk record, map shifts to correct atom names
		set editIx 0
		foreach field $mySpectra(connect,PKID,$iPos) {
			set xpkix $mySpectra(connect,XPK_INDEX,$field); 
			set pkId [lindex $pkList $editIx]
			if { $pkId == $snbConst(NO_VAL) } {
				continue
			}
			set iPeak $myPeak($xpkix,index,$pkId)
			set peakVals $myPeak($xpkix,origPeak,$iPeak)
			set atomIx 0
			foreach atomName $mySpectra(connect,ATOMS,$editIx) {
				set varSubmit($iPos,$matchNum,$atomName) [lindex $peakVals $atomIx]
				incr atomIx
			}
			incr editIx
		}

		incr matchNum 
	}
	return $matchNum
}

######################################################################
#
# propose atom values based on peakids in the menu id
#
proc submit_ids_menu { list y } {
	global snbVar snbConst varSubmit mySpectra

	# get the user's choice
	set ix $varSubmit(resIx)
	set select [expr 0 + [$varSubmit($ix,pkIdList) nearest $y]]

	# show proposed atom values
	foreach atomName $mySpectra(atoms,names) {
		set varSubmit($ix,atom,$atomName) $varSubmit($ix,$select,$atomName)
	}

	# show current peaks selected
	set aList [lindex $varSubmit($ix,totList) $select]
	set count 0
	foreach peakName $mySpectra(connect,ATOMS,TITLE) {
		set varSubmit($ix,peak,$peakName) [lindex $aList $count]
		incr count
	}
}

######################################################################
#
# clear atom values
#
proc do_clear_atoms { } {
	global snbVar snbConst varSubmit mySpectra

	set ix $varSubmit(resIx)
	foreach atomName $mySpectra(atoms,names) {
		set varSubmit($ix,atom,$atomName) ""
	}
	foreach peakName $mySpectra(connect,ATOMS,TITLE) {
		set varSubmit($ix,peak,$peakName) ""
	}
}

######################################################################
#
# propose atom values based on peakids in the menu id
#
proc submit_ids_kbd { list y } {
	global snbVar snbConst varSubmit myConnect mySpectra

	# get the user's choice
	set select [expr 0 + [$varSubmit($varSubmit(resIx),pkIdList) nearest $y]]
	set cix $varSubmit(cix,$select)

	foreach atomName $mySpectra(atoms,names) {
		set field $mySpectra(connect,$atomName,$varSubmit(resIx))
		set varSubmit($atomName) $myConnect($cix,1,$field) 
	}
}

######################################################################
#
# Screen 4:  save connect 
#
proc frame_submit_save { f } {
	global snbVar snbConst varSubmit

	pack [frame $f] -fill x -expand true -anchor n -padx 20
	label $f.title -text "Save Connectivity (step 4 of 4)"
	pack $f.title

	pack [frame $f.1] -fill x -expand true
	message $f.1.intro -text \
		"Select the save button to write the connectivity to $snbVar(fn,submit)"
	pack $f.1.intro

	pack [frame $f.ref0] -pady 10
	pack [frame $f.ref1] -pady 10
	u_command_entry $f.ref0 "RefId(i-1):" 12 \
			"" -textvariable varSubmit(cRec,0) -width 4 
	u_command_entry $f.ref1 "RefId(i):" 12 \
			"" -textvariable varSubmit(cRec,1) -width 4 

	pack [frame $f.buttons] -pady 15 -padx 10
	pack [button $f.buttons.cont -text "Save" \
		-padx $snbVar(snbwin,padx) \
		-command "save_submit"] -side left
	pack [button $f.buttons.cancel -text "Abort" \
		-padx $snbVar(snbwin,padx) \
		-command "usnb_kill_win_save $varSubmit(win)"] -side left

	pack [frame $f.2] -fill x -expand true 
	message $f.2.intro -text \
		"*** Reminder ***  If you create a connection and it does not appear in smartnotebook then you probably need to turn up the 'Standard deviation of assignment' (eg, Best Fit Button)!"
	pack $f.2.intro
}


######################################################################
#
# save connection record to output file
#
proc save_submit { } {
    global snbVar snbConst varSubmit mySpectra

	if {[ catch { set fp [open $snbVar(fn,submit) a ] } err]} {
		usnb_err "Cannot open $snbVar(fn,submit) for writing"
	}

	set line ""
	set line "$line[format " %3d %3d" $varSubmit(cRec,0) $varSubmit(cRec,1)]"
	set line "$line[format " %3s %4.1f " uxx 1.0]"

	# add peak ids
	foreach ix "0 1" {
		foreach peakName $mySpectra(connect,ATOMS,TITLE) {
			set line "$line[format " %3d" $varSubmit($ix,peak,$peakName)]"
		}
	}

	# perhaps a more complex routine would allow user to set this
	# part of the connectivity record
	set line "$line -99 -99 -99 -99 -99    0     1.0     1.0"

	# add atom values
	foreach ix "0 1" {
		foreach atomName $mySpectra(atoms,names) {
			set line "$line[format " %8.2f" $varSubmit($ix,atom,$atomName)]"
		}
	}

	# write to the file
	puts $fp $line 
	close $fp

	# update our connection lists
	snb_read_data $snbConst(E_SEQ)
	usnb_kill_win_save $varSubmit(win)
}
